@@ -1,6 +1,10 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * scanner.c
  *
@@ -11,7 +15,7 @@
  * C identifiers up into keywords and SWIG directives.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cscanner_c[] = "$Id: cscanner.c 10511 2008-05-30 16:11:27Z olly $";
+char cvsroot_cscanner_c[] = "$Id: cscanner.c 11876 2010-02-27 23:53:33Z wsfulton $";
 
 #include "cparse.h"
 #include "parser.h"
@@ -22,7 +26,10 @@ char cvsroot_cscanner_c[] = "$Id: cscanner.c 10511 2008-05-30 16:11:27Z olly $";
 static Scanner *scan = 0;
 
 /* Global string containing C code. Used by the parser to grab code blocks */
-DOHString *scanner_ccode = 0;
+String *scanner_ccode = 0;
+
+/* The main file being parsed */
+static String *main_input_file = 0;
 
 /* Error reporting/location information */
 int     cparse_line = 1;
@@ -132,7 +139,7 @@ static void scanner_locator(String *loc) {
 	  break;
 	Putc(c, fn);
       }
-      /*  Printf(stderr,"location: %s:%d\n",cparse_file,cparse_line); */
+      /*  Swig_diagnostic(cparse_file, cparse_line, "Scanner_set_location\n"); */
       Scanner_set_location(scan,cparse_file,cparse_line);
       Delete(fn);
     }
@@ -254,17 +261,15 @@ void skip_decl(void) {
  * Lexical scanner.
  * ------------------------------------------------------------------------- */
 
-int yylook(void) {
+static int yylook(void) {
 
   int tok = 0;
 
   while (1) {
     if ((tok = Scanner_token(scan)) == 0)
       return 0;
-    if (tok == SWIG_TOKEN_ERROR) {
-      Swig_error(Scanner_file(scan), Scanner_errline(scan), Scanner_errmsg(scan));
-      continue;
-    }
+    if (tok == SWIG_TOKEN_ERROR)
+      return 0;
     cparse_start_line = Scanner_start_line(scan);
     cparse_line = Scanner_line(scan);
     cparse_file = Scanner_file(scan);
@@ -409,6 +414,9 @@ int yylook(void) {
     case SWIG_TOKEN_FLOAT:
       return NUM_FLOAT;
       
+    case SWIG_TOKEN_BOOL:
+      return NUM_BOOL;
+      
     case SWIG_TOKEN_POUND:
       Scanner_skip_line(scan);
       yylval.id = Swig_copy_string(Char(Scanner_text(scan)));
@@ -441,7 +449,7 @@ int yylook(void) {
 
 static int check_typedef = 0;
 
-void scanner_set_location(String_or_char *file, int line) {
+void scanner_set_location(String *file, int line) {
   Scanner_set_location(scan,file,line-1);
 }
 
@@ -467,6 +475,14 @@ void scanner_next_token(int tok) {
   next_token = tok;
 }
 
+void scanner_set_main_input_file(String *file) {
+  main_input_file = file;
+}
+
+String *scanner_get_main_input_file() {
+  return main_input_file;
+}
+
 /* ----------------------------------------------------------------------------
  * int yylex()
  *
@@ -490,7 +506,7 @@ int yylex(void) {
 
   l = yylook();
 
-  /*   Printf(stdout, "%s:%d:::%d: '%s'\n", cparse_file, cparse_line, l, Scanner_text(scan)); */
+  /*   Swig_diagnostic(cparse_file, cparse_line, ":::%d: '%s'\n", l, Scanner_text(scan)); */
 
   if (l == NONID) {
     last_id = 1;
@@ -516,6 +532,7 @@ int yylex(void) {
   case NUM_UNSIGNED:
   case NUM_LONGLONG:
   case NUM_ULONGLONG:
+  case NUM_BOOL:
     if (l == NUM_INT)
       yylval.dtype.type = T_INT;
     if (l == NUM_FLOAT)
@@ -530,6 +547,8 @@ int yylex(void) {
       yylval.dtype.type = T_LONGLONG;
     if (l == NUM_ULONGLONG)
       yylval.dtype.type = T_ULONGLONG;
+    if (l == NUM_BOOL)
+      yylval.dtype.type = T_BOOL;
     yylval.dtype.val = NewString(Scanner_text(scan));
     yylval.dtype.bitfield = 0;
     yylval.dtype.throws = 0;
@@ -690,10 +709,22 @@ int yylex(void) {
 		termtoken = SWIG_TOKEN_LPAREN;
 		termvalue = "(";
 		break;
-	      } else if (nexttok == SWIG_TOKEN_SEMI) {
+              } else if (nexttok == SWIG_TOKEN_CODEBLOCK) {
+                termtoken = SWIG_TOKEN_CODEBLOCK;
+                termvalue = Char(Scanner_text(scan));
+                break;
+              } else if (nexttok == SWIG_TOKEN_LBRACE) {
+                termtoken = SWIG_TOKEN_LBRACE;
+                termvalue = "{";
+                break;
+              } else if (nexttok == SWIG_TOKEN_SEMI) {
 		termtoken = SWIG_TOKEN_SEMI;
 		termvalue = ";";
 		break;
+              } else if (nexttok == SWIG_TOKEN_STRING) {
+		termtoken = SWIG_TOKEN_STRING;
+                termvalue = Swig_copy_string(Char(Scanner_text(scan)));
+		break;
 	      } else if (nexttok == SWIG_TOKEN_ID) {
 		if (needspace) {
 		  Append(s," ");
@@ -859,8 +890,14 @@ int yylex(void) {
 	return (INLINE);
       if (strcmp(yytext, "%typemap") == 0)
 	return (TYPEMAP);
-      if (strcmp(yytext, "%feature") == 0)
+      if (strcmp(yytext, "%feature") == 0) {
+        /* The rename_active indicates we don't need the information of the 
+         * following function's return type. This applied for %rename, so do
+         * %feature. 
+         */
+        rename_active = 1;
 	return (FEATURE);
+      }
       if (strcmp(yytext, "%except") == 0)
 	return (EXCEPT);
       if (strcmp(yytext, "%importfile") == 0)
