@@ -54,81 +54,81 @@
      NUM_ULONG = 270,
      NUM_LONGLONG = 271,
      NUM_ULONGLONG = 272,
-     TYPEDEF = 273,
-     TYPE_INT = 274,
-     TYPE_UNSIGNED = 275,
-     TYPE_SHORT = 276,
-     TYPE_LONG = 277,
-     TYPE_FLOAT = 278,
-     TYPE_DOUBLE = 279,
-     TYPE_CHAR = 280,
-     TYPE_WCHAR = 281,
-     TYPE_VOID = 282,
-     TYPE_SIGNED = 283,
-     TYPE_BOOL = 284,
-     TYPE_COMPLEX = 285,
-     TYPE_TYPEDEF = 286,
-     TYPE_RAW = 287,
-     TYPE_NON_ISO_INT8 = 288,
-     TYPE_NON_ISO_INT16 = 289,
-     TYPE_NON_ISO_INT32 = 290,
-     TYPE_NON_ISO_INT64 = 291,
-     LPAREN = 292,
-     RPAREN = 293,
-     COMMA = 294,
-     SEMI = 295,
-     EXTERN = 296,
-     INIT = 297,
-     LBRACE = 298,
-     RBRACE = 299,
-     PERIOD = 300,
-     CONST_QUAL = 301,
-     VOLATILE = 302,
-     REGISTER = 303,
-     STRUCT = 304,
-     UNION = 305,
-     EQUAL = 306,
-     SIZEOF = 307,
-     MODULE = 308,
-     LBRACKET = 309,
-     RBRACKET = 310,
-     ILLEGAL = 311,
-     CONSTANT = 312,
-     NAME = 313,
-     RENAME = 314,
-     NAMEWARN = 315,
-     EXTEND = 316,
-     PRAGMA = 317,
-     FEATURE = 318,
-     VARARGS = 319,
-     ENUM = 320,
-     CLASS = 321,
-     TYPENAME = 322,
-     PRIVATE = 323,
-     PUBLIC = 324,
-     PROTECTED = 325,
-     COLON = 326,
-     STATIC = 327,
-     VIRTUAL = 328,
-     FRIEND = 329,
-     THROW = 330,
-     CATCH = 331,
-     EXPLICIT = 332,
-     USING = 333,
-     NAMESPACE = 334,
-     NATIVE = 335,
-     INLINE = 336,
-     TYPEMAP = 337,
-     EXCEPT = 338,
-     ECHO = 339,
-     APPLY = 340,
-     CLEAR = 341,
-     SWIGTEMPLATE = 342,
-     FRAGMENT = 343,
-     WARN = 344,
-     LESSTHAN = 345,
-     GREATERTHAN = 346,
-     MODULO = 347,
+     NUM_BOOL = 273,
+     TYPEDEF = 274,
+     TYPE_INT = 275,
+     TYPE_UNSIGNED = 276,
+     TYPE_SHORT = 277,
+     TYPE_LONG = 278,
+     TYPE_FLOAT = 279,
+     TYPE_DOUBLE = 280,
+     TYPE_CHAR = 281,
+     TYPE_WCHAR = 282,
+     TYPE_VOID = 283,
+     TYPE_SIGNED = 284,
+     TYPE_BOOL = 285,
+     TYPE_COMPLEX = 286,
+     TYPE_TYPEDEF = 287,
+     TYPE_RAW = 288,
+     TYPE_NON_ISO_INT8 = 289,
+     TYPE_NON_ISO_INT16 = 290,
+     TYPE_NON_ISO_INT32 = 291,
+     TYPE_NON_ISO_INT64 = 292,
+     LPAREN = 293,
+     RPAREN = 294,
+     COMMA = 295,
+     SEMI = 296,
+     EXTERN = 297,
+     INIT = 298,
+     LBRACE = 299,
+     RBRACE = 300,
+     PERIOD = 301,
+     CONST_QUAL = 302,
+     VOLATILE = 303,
+     REGISTER = 304,
+     STRUCT = 305,
+     UNION = 306,
+     EQUAL = 307,
+     SIZEOF = 308,
+     MODULE = 309,
+     LBRACKET = 310,
+     RBRACKET = 311,
+     ILLEGAL = 312,
+     CONSTANT = 313,
+     NAME = 314,
+     RENAME = 315,
+     NAMEWARN = 316,
+     EXTEND = 317,
+     PRAGMA = 318,
+     FEATURE = 319,
+     VARARGS = 320,
+     ENUM = 321,
+     CLASS = 322,
+     TYPENAME = 323,
+     PRIVATE = 324,
+     PUBLIC = 325,
+     PROTECTED = 326,
+     COLON = 327,
+     STATIC = 328,
+     VIRTUAL = 329,
+     FRIEND = 330,
+     THROW = 331,
+     CATCH = 332,
+     EXPLICIT = 333,
+     USING = 334,
+     NAMESPACE = 335,
+     NATIVE = 336,
+     INLINE = 337,
+     TYPEMAP = 338,
+     EXCEPT = 339,
+     ECHO = 340,
+     APPLY = 341,
+     CLEAR = 342,
+     SWIGTEMPLATE = 343,
+     FRAGMENT = 344,
+     WARN = 345,
+     LESSTHAN = 346,
+     GREATERTHAN = 347,
      DELETE_KW = 348,
      LESSTHANOREQUALTO = 349,
      GREATERTHANOREQUALTO = 350,
@@ -156,7 +156,7 @@
      LSHIFT = 372,
      MINUS = 373,
      PLUS = 374,
-     MODULUS = 375,
+     MODULO = 375,
      SLASH = 376,
      STAR = 377,
      LNOT = 378,
@@ -181,81 +181,81 @@
 #define NUM_ULONG 270
 #define NUM_LONGLONG 271
 #define NUM_ULONGLONG 272
-#define TYPEDEF 273
-#define TYPE_INT 274
-#define TYPE_UNSIGNED 275
-#define TYPE_SHORT 276
-#define TYPE_LONG 277
-#define TYPE_FLOAT 278
-#define TYPE_DOUBLE 279
-#define TYPE_CHAR 280
-#define TYPE_WCHAR 281
-#define TYPE_VOID 282
-#define TYPE_SIGNED 283
-#define TYPE_BOOL 284
-#define TYPE_COMPLEX 285
-#define TYPE_TYPEDEF 286
-#define TYPE_RAW 287
-#define TYPE_NON_ISO_INT8 288
-#define TYPE_NON_ISO_INT16 289
-#define TYPE_NON_ISO_INT32 290
-#define TYPE_NON_ISO_INT64 291
-#define LPAREN 292
-#define RPAREN 293
-#define COMMA 294
-#define SEMI 295
-#define EXTERN 296
-#define INIT 297
-#define LBRACE 298
-#define RBRACE 299
-#define PERIOD 300
-#define CONST_QUAL 301
-#define VOLATILE 302
-#define REGISTER 303
-#define STRUCT 304
-#define UNION 305
-#define EQUAL 306
-#define SIZEOF 307
-#define MODULE 308
-#define LBRACKET 309
-#define RBRACKET 310
-#define ILLEGAL 311
-#define CONSTANT 312
-#define NAME 313
-#define RENAME 314
-#define NAMEWARN 315
-#define EXTEND 316
-#define PRAGMA 317
-#define FEATURE 318
-#define VARARGS 319
-#define ENUM 320
-#define CLASS 321
-#define TYPENAME 322
-#define PRIVATE 323
-#define PUBLIC 324
-#define PROTECTED 325
-#define COLON 326
-#define STATIC 327
-#define VIRTUAL 328
-#define FRIEND 329
-#define THROW 330
-#define CATCH 331
-#define EXPLICIT 332
-#define USING 333
-#define NAMESPACE 334
-#define NATIVE 335
-#define INLINE 336
-#define TYPEMAP 337
-#define EXCEPT 338
-#define ECHO 339
-#define APPLY 340
-#define CLEAR 341
-#define SWIGTEMPLATE 342
-#define FRAGMENT 343
-#define WARN 344
-#define LESSTHAN 345
-#define GREATERTHAN 346
-#define MODULO 347
+#define NUM_BOOL 273
+#define TYPEDEF 274
+#define TYPE_INT 275
+#define TYPE_UNSIGNED 276
+#define TYPE_SHORT 277
+#define TYPE_LONG 278
+#define TYPE_FLOAT 279
+#define TYPE_DOUBLE 280
+#define TYPE_CHAR 281
+#define TYPE_WCHAR 282
+#define TYPE_VOID 283
+#define TYPE_SIGNED 284
+#define TYPE_BOOL 285
+#define TYPE_COMPLEX 286
+#define TYPE_TYPEDEF 287
+#define TYPE_RAW 288
+#define TYPE_NON_ISO_INT8 289
+#define TYPE_NON_ISO_INT16 290
+#define TYPE_NON_ISO_INT32 291
+#define TYPE_NON_ISO_INT64 292
+#define LPAREN 293
+#define RPAREN 294
+#define COMMA 295
+#define SEMI 296
+#define EXTERN 297
+#define INIT 298
+#define LBRACE 299
+#define RBRACE 300
+#define PERIOD 301
+#define CONST_QUAL 302
+#define VOLATILE 303
+#define REGISTER 304
+#define STRUCT 305
+#define UNION 306
+#define EQUAL 307
+#define SIZEOF 308
+#define MODULE 309
+#define LBRACKET 310
+#define RBRACKET 311
+#define ILLEGAL 312
+#define CONSTANT 313
+#define NAME 314
+#define RENAME 315
+#define NAMEWARN 316
+#define EXTEND 317
+#define PRAGMA 318
+#define FEATURE 319
+#define VARARGS 320
+#define ENUM 321
+#define CLASS 322
+#define TYPENAME 323
+#define PRIVATE 324
+#define PUBLIC 325
+#define PROTECTED 326
+#define COLON 327
+#define STATIC 328
+#define VIRTUAL 329
+#define FRIEND 330
+#define THROW 331
+#define CATCH 332
+#define EXPLICIT 333
+#define USING 334
+#define NAMESPACE 335
+#define NATIVE 336
+#define INLINE 337
+#define TYPEMAP 338
+#define EXCEPT 339
+#define ECHO 340
+#define APPLY 341
+#define CLEAR 342
+#define SWIGTEMPLATE 343
+#define FRAGMENT 344
+#define WARN 345
+#define LESSTHAN 346
+#define GREATERTHAN 347
 #define DELETE_KW 348
 #define LESSTHANOREQUALTO 349
 #define GREATERTHANOREQUALTO 350
@@ -283,7 +283,7 @@
 #define LSHIFT 372
 #define MINUS 373
 #define PLUS 374
-#define MODULUS 375
+#define MODULO 375
 #define SLASH 376
 #define STAR 377
 #define LNOT 378
@@ -296,7 +296,7 @@
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 1373 "parser.y"
+#line 1592 "parser.y"
 {
   char  *id;
   List  *bases;
@@ -325,7 +325,7 @@ typedef union YYSTYPE
   } decl;
   Parm         *tparms;
   struct {
-    String     *op;
+    String     *method;
     Hash       *kwargs;
   } tmap;
   struct {
