@@ -1,16 +1,18 @@
 /* -----------------------------------------------------------------------------
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
+ *
  * base.c 
  *
  *     This file contains the function entry points for dispatching methods on
  *     DOH objects.  A number of small utility functions are also included.
- *
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1999-2000.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_base_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/base.c,v 1.20 2006/11/01 23:54:50 wsfulton Exp $";
+char cvsroot_base_c[] = "$Id: base.c 11897 2010-03-04 21:45:26Z wsfulton $";
 
 #include "dohint.h"
 
@@ -28,12 +30,15 @@ void DohDelete(DOH *obj) {
 
   if (!obj)
     return;
-#if SWIG_DEBUG_DELETE
   if (!DohCheck(b)) {
+#if SWIG_DEBUG_DELETE
     fputs("DOH: Fatal error. Attempt to delete a non-doh object.\n", stderr);
     abort();
-  }
+#else
+    assert(0);
 #endif
+    return;
+  }
   if (b->flag_intern)
     return;
   assert(b->refcount > 0);
@@ -60,6 +65,15 @@ DOH *DohCopy(const DOH *obj) {
 
   if (!obj)
     return 0;
+  if (!DohCheck(b)) {
+#if SWIG_DEBUG_DELETE
+    fputs("DOH: Fatal error. Attempt to copy a non-doh object.\n", stderr);
+    abort();
+#else
+    assert(0);
+#endif
+    return 0;
+  }
   objinfo = b->type;
   if (objinfo->doh_copy) {
     DohBase *bc = (DohBase *) (objinfo->doh_copy) (b);
@@ -353,6 +367,16 @@ int DohDelattr(DOH *obj, const DOH *name) {
   return 0;
 }
 
+/* -----------------------------------------------------------------------------
+ * DohCheckattr()
+ * ----------------------------------------------------------------------------- */
+
+int DohCheckattr(DOH *obj, const DOH *name, const DOH *value) {
+  DOH *attr = Getattr(obj,name);
+  if (!attr) return 0;
+  return DohEqual(attr,value);
+}
+
 /* -----------------------------------------------------------------------------
  * DohKeys()
  * ----------------------------------------------------------------------------- */
@@ -817,7 +841,7 @@ void DohSetfile(DOH *ho, DOH *file) {
 /* -----------------------------------------------------------------------------
  * DohGetFile()
  * ----------------------------------------------------------------------------- */
-DOH *DohGetfile(DOH *ho) {
+DOH *DohGetfile(const DOH *ho) {
   DohBase *h = (DohBase *) ho;
   DohObjInfo *objinfo;
   if (!h)
@@ -844,7 +868,7 @@ void DohSetline(DOH *ho, int l) {
 /* -----------------------------------------------------------------------------
  * DohGetLine()
  * ----------------------------------------------------------------------------- */
-int DohGetline(DOH *ho) {
+int DohGetline(const DOH *ho) {
   DohBase *h = (DohBase *) ho;
   DohObjInfo *objinfo;
   if (!h)
