@@ -1,14 +1,14 @@
 /* -----------------------------------------------------------------------------
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
+ *
  * doh.h
  *
  *     This file describes of the externally visible functions in DOH.
- *
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1999-2000.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.
- *
- * $Header: /cvsroot/swig/SWIG/Source/DOH/doh.h,v 1.21 2006/02/19 22:16:26 wsfulton Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef _DOH_H
@@ -73,17 +73,6 @@
 #define DohGetc            DOH_NAMESPACE(Getc)
 #define DohPutc            DOH_NAMESPACE(Putc)
 #define DohUngetc          DOH_NAMESPACE(Ungetc)
-
-#define DohStringPutc      DOH_NAMESPACE(StringPutc)
-#define DohStringGetc      DOH_NAMESPACE(StringGetc)
-#define DohStringUngetc    DOH_NAMESPACE(StringUngetc)
-#define DohStringAppend    DOH_NAMESPACE(StringAppend)
-#define DohStringLen       DOH_NAMESPACE(StringLen)
-#define DohStringChar      DOH_NAMESPACE(StringChar)
-#define DohStringEqual     DOH_NAMESPACE(StringEqual)
-
-
-
 #define DohGetline         DOH_NAMESPACE(Getline)
 #define DohSetline         DOH_NAMESPACE(Setline)
 #define DohGetfile         DOH_NAMESPACE(Getfile)
@@ -118,8 +107,6 @@
 #define DohCopyto          DOH_NAMESPACE(Copyto)
 #define DohNewList         DOH_NAMESPACE(NewList)
 #define DohNewHash         DOH_NAMESPACE(NewHash)
-#define DohHashGetAttr     DOH_NAMESPACE(HashGetAttr)
-#define DohHashCheckAttr   DOH_NAMESPACE(HashCheckAttr)
 #define DohNewVoid         DOH_NAMESPACE(NewVoid)
 #define DohSplit           DOH_NAMESPACE(Split)
 #define DohSplitLines      DOH_NAMESPACE(SplitLines)
@@ -157,6 +144,9 @@ typedef void DOH;
 #define DOHString_or_char  DOH
 #define DOHObj_or_char     DOH
 
+typedef const DOHString_or_char * const_String_or_char_ptr;
+typedef const DOHString_or_char * DOHconst_String_or_char_ptr;
+
 #define DOH_BEGIN          -1
 #define DOH_END            -2
 #define DOH_CUR            -3
@@ -165,11 +155,11 @@ typedef void DOH;
 /* Iterator objects */
 
 typedef struct {
-  void   *key;             /* Current key (if any)       */
-  void   *item;            /* Current item               */
-  void   *object;          /* Object being iterated over */
-  void   *_current;        /* Internal use */
-  int    _index;           /* Internal use */
+  void *key;			/* Current key (if any)       */
+  void *item;			/* Current item               */
+  void *object;			/* Object being iterated over */
+  void *_current;		/* Internal use */
+  int _index;			/* Internal use */
 } DohIterator;
 
 /* Memory management */
@@ -184,128 +174,113 @@ typedef struct {
 #define DohFree free
 #endif
 
-extern int           DohCheck(const DOH *ptr);           /* Check if a DOH object */
-extern void          DohIntern(DOH *);                   /* Intern an object      */
+extern int DohCheck(const DOH *ptr);	/* Check if a DOH object */
+extern void DohIntern(DOH *);	/* Intern an object      */
 
 /* Basic object methods.  Common to most objects */
 
-extern void          DohDelete(DOH *obj);                /* Delete an object      */
-extern DOH          *DohCopy(const DOH *obj);
-extern void          DohClear(DOH *obj);
-extern DOHString    *DohStr(const DOH *obj);
-extern void         *DohData(const DOH *obj);
-extern int           DohDump(const DOH *obj, DOHFile *out);
-extern int           DohLen(const DOH *obj);
-extern int           DohHashval(const DOH *obj);
-extern int           DohCmp(const DOH *obj1, const DOH *obj2);
-extern int           DohEqual(const DOH *obj1, const DOH *obj2);
-extern void          DohIncref(DOH *obj);
+extern void DohDelete(DOH *obj);	/* Delete an object      */
+extern DOH *DohCopy(const DOH *obj);
+extern void DohClear(DOH *obj);
+extern DOHString *DohStr(const DOH *obj);
+extern void *DohData(const DOH *obj);
+extern int DohDump(const DOH *obj, DOHFile * out);
+extern int DohLen(const DOH *obj);
+extern int DohHashval(const DOH *obj);
+extern int DohCmp(const DOH *obj1, const DOH *obj2);
+extern int DohEqual(const DOH *obj1, const DOH *obj2);
+extern void DohIncref(DOH *obj);
 
 /* Mapping methods */
 
-extern DOH          *DohGetattr(DOH *obj, const DOHString_or_char *name);
-extern int           DohSetattr(DOH *obj, const DOHString_or_char *name, const DOHObj_or_char *value);
-extern int           DohDelattr(DOH *obj, const DOHString_or_char *name);
-extern DOH          *DohKeys(DOH *obj);
-extern int           DohGetInt(DOH *obj, const DOHString_or_char *name);
-extern void          DohSetInt(DOH *obj, const DOHString_or_char *name, int);
-extern double        DohGetDouble(DOH *obj, const DOHString_or_char *name);
-extern void          DohSetDouble(DOH *obj, const DOHString_or_char *name, double);
-extern char         *DohGetChar(DOH *obj, const DOHString_or_char *name);
-extern void          DohSetChar(DOH *obj, const DOH *name, char *value);
-extern void         *DohGetFlagAttr(DOH *obj, const DOHString_or_char *name);
-extern int           DohGetFlag(DOH *obj, const DOHString_or_char *name);
-extern void          DohSetFlagAttr(DOH *obj, const DOHString_or_char *name, const DOHString_or_char *attr);
-extern void          DohSetFlag(DOH *obj, const DOHString_or_char *name);
-extern void         *DohGetVoid(DOH *obj, const DOHString_or_char *name);
-extern void          DohSetVoid(DOH *obj, const DOHString_or_char *name, void *value);
+extern DOH *DohGetattr(DOH *obj, const DOHString_or_char *name);
+extern int DohSetattr(DOH *obj, const DOHString_or_char *name, const DOHObj_or_char * value);
+extern int DohDelattr(DOH *obj, const DOHString_or_char *name);
+extern int DohCheckattr(DOH *obj, const DOHString_or_char *name, const DOHString_or_char *value);
+extern DOH *DohKeys(DOH *obj);
+extern int DohGetInt(DOH *obj, const DOHString_or_char *name);
+extern void DohSetInt(DOH *obj, const DOHString_or_char *name, int);
+extern double DohGetDouble(DOH *obj, const DOHString_or_char *name);
+extern void DohSetDouble(DOH *obj, const DOHString_or_char *name, double);
+extern char *DohGetChar(DOH *obj, const DOHString_or_char *name);
+extern void DohSetChar(DOH *obj, const DOH *name, char *value);
+extern void *DohGetFlagAttr(DOH *obj, const DOHString_or_char *name);
+extern int DohGetFlag(DOH *obj, const DOHString_or_char *name);
+extern void DohSetFlagAttr(DOH *obj, const DOHString_or_char *name, const DOHString_or_char *attr);
+extern void DohSetFlag(DOH *obj, const DOHString_or_char *name);
+extern void *DohGetVoid(DOH *obj, const DOHString_or_char *name);
+extern void DohSetVoid(DOH *obj, const DOHString_or_char *name, void *value);
 
 /* Sequence methods */
 
-extern DOH          *DohGetitem(DOH *obj, int index);
-extern int           DohSetitem(DOH *obj, int index, const DOHObj_or_char *value);
-extern int           DohDelitem(DOH *obj, int index);
-extern int           DohInsertitem(DOH *obj, int index, const DOHObj_or_char *value);
-extern int           DohDelslice(DOH *obj, int sindex, int eindex);
+extern DOH *DohGetitem(DOH *obj, int index);
+extern int DohSetitem(DOH *obj, int index, const DOHObj_or_char * value);
+extern int DohDelitem(DOH *obj, int index);
+extern int DohInsertitem(DOH *obj, int index, const DOHObj_or_char * value);
+extern int DohDelslice(DOH *obj, int sindex, int eindex);
 
 /* File methods */
 
-extern int           DohWrite(DOHFile *obj, void *buffer, int length);
-extern int           DohRead(DOHFile *obj, void *buffer, int length);
-extern int           DohSeek(DOHFile *obj, long offset, int whence);
-extern long          DohTell(DOHFile *obj);
-extern int           DohGetc(DOHFile *obj);
-extern int           DohPutc(int ch, DOHFile *obj);
-extern int           DohUngetc(int ch, DOHFile *obj);
+extern int DohWrite(DOHFile * obj, void *buffer, int length);
+extern int DohRead(DOHFile * obj, void *buffer, int length);
+extern int DohSeek(DOHFile * obj, long offset, int whence);
+extern long DohTell(DOHFile * obj);
+extern int DohGetc(DOHFile * obj);
+extern int DohPutc(int ch, DOHFile * obj);
+extern int DohUngetc(int ch, DOHFile * obj);
 
 
 
 /* Iterators */
-extern DohIterator   DohFirst(DOH *obj);
-extern DohIterator   DohNext(DohIterator x);
+extern DohIterator DohFirst(DOH *obj);
+extern DohIterator DohNext(DohIterator x);
 
 /* Positional */
 
-extern int           DohGetline(DOH *obj);
-extern void          DohSetline(DOH *obj, int line);
-extern DOH          *DohGetfile(DOH *obj);
-extern void          DohSetfile(DOH *obj, DOH *file);
+extern int DohGetline(const DOH *obj);
+extern void DohSetline(DOH *obj, int line);
+extern DOH *DohGetfile(const DOH *obj);
+extern void DohSetfile(DOH *obj, DOH *file);
 
   /* String Methods */
 
-extern int           DohReplace(DOHString *src, const DOHString_or_char *token, const DOHString_or_char *rep, int flags);
-extern void          DohChop(DOHString *src);
-
-extern int DohString_putc(DOH *so, int ch);
-extern int DohString_getc(DOH *so);
-extern int DohString_ungetc(DOH *so, int ch);
-extern void DohString_append(DOH *so, DOH *str);
-extern int DohString_len(DOH *s1);
-extern char *DohString_char(DOH *s1);
-extern int DohString_equal(DOH *s1, DOH *s2);
-
-#define DohStringPutc(ch,so)    DohString_putc(so, ch)
-#define DohStringGetc(so)       DohString_getc(so)
-#define DohStringUngetc(ch,so)  DohString_ungetc(so, ch)
-#define DohStringAppend(so,str) DohString_append(so, (DOH*)str)
-#define DohStringLen(so)        DohString_len((DOH*)so)
-#define DohStringChar(so)       DohString_char(so)
-#define DohStringEqual(s1,s2)   DohString_equal((DOH *)s1, (DOH *)s2)
+extern int DohReplace(DOHString * src, const DOHString_or_char *token, const DOHString_or_char *rep, int flags);
+extern void DohChop(DOHString * src);
 
 /* Meta-variables */
-extern DOH          *DohGetmeta(DOH *, const DOH *);
-extern int           DohSetmeta(DOH *, const DOH *, const DOH *value);
-extern int           DohDelmeta(DOH *, const DOH *);
+extern DOH *DohGetmeta(DOH *, const DOH *);
+extern int DohSetmeta(DOH *, const DOH *, const DOH *value);
+extern int DohDelmeta(DOH *, const DOH *);
 
   /* Utility functions */
 
-extern void          DohEncoding(char *name, DOH *(*fn)(DOH *s));
-extern int           DohPrintf(DOHFile *obj, const char *format, ...);
-extern int           DohvPrintf(DOHFile *obj, const char *format, va_list ap);
-extern int           DohPrintv(DOHFile *obj, ...);
-extern DOH          *DohReadline(DOHFile *in);
+extern void DohEncoding(char *name, DOH *(*fn) (DOH *s));
+extern int DohPrintf(DOHFile * obj, const char *format, ...);
+extern int DohvPrintf(DOHFile * obj, const char *format, va_list ap);
+extern int DohPrintv(DOHFile * obj, ...);
+extern DOH *DohReadline(DOHFile * in);
 
   /* Miscellaneous */
 
-extern int           DohIsMapping(const DOH *obj);
-extern int           DohIsSequence(const DOH *obj);
-extern int           DohIsString(const DOH *obj);
-extern int           DohIsFile(const DOH *obj);
+extern int DohIsMapping(const DOH *obj);
+extern int DohIsSequence(const DOH *obj);
+extern int DohIsString(const DOH *obj);
+extern int DohIsFile(const DOH *obj);
 
-extern void          DohSetmark(DOH *obj, int x);
-extern int           DohGetmark(DOH *obj);
+extern void DohSetmark(DOH *obj, int x);
+extern int DohGetmark(DOH *obj);
 
 /* -----------------------------------------------------------------------------
  * Strings.
  * ----------------------------------------------------------------------------- */
 
-extern DOHString    *DohNewStringEmpty();
-extern DOHString    *DohNewString(const DOH *c);
-extern DOHString    *DohNewStringWithSize(const DOH *c, int len);
-extern DOHString    *DohNewStringf(const DOH *fmt, ...);
+extern DOHString *DohNewStringEmpty(void);
+extern DOHString *DohNewString(const DOH *c);
+extern DOHString *DohNewStringWithSize(const DOH *c, int len);
+extern DOHString *DohNewStringf(const DOH *fmt, ...);
 
-extern int   DohStrcmp(const DOHString_or_char *s1, const DOHString_or_char *s2);
-extern int   DohStrncmp(const DOHString_or_char *s1, const DOHString_or_char *s2, int n);
+extern int DohStrcmp(const DOHString_or_char *s1, const DOHString_or_char *s2);
+extern int DohStrncmp(const DOHString_or_char *s1, const DOHString_or_char *s2, int n);
 extern char *DohStrstr(const DOHString_or_char *s1, const DOHString_or_char *s2);
 extern char *DohStrchr(const DOHString_or_char *s1, int ch);
 
@@ -325,39 +300,37 @@ extern char *DohStrchr(const DOHString_or_char *s1, int ch);
  * Files
  * ----------------------------------------------------------------------------- */
 
-extern DOHFile *DohNewFile(DOH *file, const char *mode);
+extern DOHFile *DohNewFile(DOH *filename, const char *mode, DOHList *outfiles);
 extern DOHFile *DohNewFileFromFile(FILE *f);
 extern DOHFile *DohNewFileFromFd(int fd);
-extern void     DohFileErrorDisplay(DOHString *filename);
-extern int      DohClose(DOH *file);
-extern int      DohCopyto(DOHFile *input, DOHFile *output);
+extern void DohFileErrorDisplay(DOHString * filename);
+extern int DohClose(DOH *file);
+extern int DohCopyto(DOHFile * input, DOHFile * output);
 
 
 /* -----------------------------------------------------------------------------
  * List
  * ----------------------------------------------------------------------------- */
 
-extern DOHList  *DohNewList();
-extern void DohSortList(DOH *lo, int (*cmp)(const DOH *, const DOH *));
+extern DOHList *DohNewList(void);
+extern void DohSortList(DOH *lo, int (*cmp) (const DOH *, const DOH *));
 
 /* -----------------------------------------------------------------------------
  * Hash
  * ----------------------------------------------------------------------------- */
 
-extern DOHHash   *DohNewHash();
-extern DOH       *DohHashGetAttr(DOH *hash, const DOH *key);
-extern int       DohHashCheckAttr(DOH *hash, DOH *key, DOH *value);
+extern DOHHash *DohNewHash(void);
 
 /* -----------------------------------------------------------------------------
  * Void
  * ----------------------------------------------------------------------------- */
 
-extern DOHVoid  *DohNewVoid(void *ptr, void (*del)(void *));
-extern DOHList  *DohSplit(DOHFile *input, char ch, int nsplits);
-extern DOHList  *DohSplitLines(DOHFile *input);
-extern DOH      *DohNone;
+extern DOHVoid *DohNewVoid(void *ptr, void (*del) (void *));
+extern DOHList *DohSplit(DOHFile * input, char ch, int nsplits);
+extern DOHList *DohSplitLines(DOHFile * input);
+extern DOH *DohNone;
 
-extern void      DohMemoryDebug(void);
+extern void DohMemoryDebug(void);
 
 #ifndef DOH_LONG_NAMES
 /* Macros to invoke the above functions.  Includes the location of
@@ -371,6 +344,7 @@ extern void      DohMemoryDebug(void);
 #define Getattr            DohGetattr
 #define Setattr            DohSetattr
 #define Delattr            DohDelattr
+#define Checkattr          DohCheckattr
 #define Hashval            DohHashval
 #define Getitem            DohGetitem
 #define Setitem            DohSetitem
@@ -398,13 +372,13 @@ extern void      DohMemoryDebug(void);
 #define Putc               DohPutc
 #define Ungetc             DohUngetc
 
-#define StringPutc         DohStringPutc
-#define StringGetc         DohStringGetc
-#define StringUngetc       DohStringUngetc
-#define StringAppend       DohStringAppend
-#define StringLen          DohStringLen
-#define StringChar         DohStringChar
-#define StringEqual        DohStringEqual
+/* #define StringPutc         DohStringPutc */
+/* #define StringGetc         DohStringGetc */
+/* #define StringUngetc       DohStringUngetc */
+/* #define StringAppend       Append */
+/* #define StringLen          DohStringLen */
+/* #define StringChar         DohStringChar */
+/* #define StringEqual        DohStringEqual */
 
 #define Close              DohClose
 #define vPrintf            DohvPrintf
@@ -433,8 +407,6 @@ extern void      DohMemoryDebug(void);
 #define NewStringWithSize  DohNewStringWithSize
 #define NewStringf         DohNewStringf
 #define NewHash            DohNewHash
-#define HashGetAttr        DohHashGetAttr
-#define HashCheckAttr      DohHashCheckAttr
 #define NewList            DohNewList
 #define NewFile            DohNewFile
 #define NewFileFromFile    DohNewFileFromFile
@@ -467,8 +439,4 @@ extern void      DohMemoryDebug(void);
 #define NIL  (char *) NULL
 
 
-#endif /* DOH_H */
-
-
-
-
+#endif				/* DOH_H */
