@@ -1,16 +1,18 @@
 /* ----------------------------------------------------------------------------- 
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
+ *
  * file.c
  *
  *     This file implements a file-like object that can be built around an 
  *     ordinary FILE * or integer file descriptor.
- * 
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1999-2000.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_file_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/file.c,v 1.7 2006/11/01 23:54:50 wsfulton Exp $";
+char cvsroot_file_c[] = "$Id: file.c 11876 2010-02-27 23:53:33Z wsfulton $";
 
 #include "dohint.h"
 
@@ -139,7 +141,7 @@ static int File_getc(DOH *fo) {
     return fgetc(f->filep);
   } else if (f->fd) {
 #ifdef DOH_INTFILE
-    char c;
+    unsigned char c;
     if (read(f->fd, &c, 1) < 0)
       return EOF;
     return c;
@@ -228,15 +230,16 @@ static DohObjInfo DohFileType = {
  * NewFile()
  *
  * Create a new file from a given filename and mode.
+ * If newfiles is non-zero, the filename is added to the list of new files.
  * ----------------------------------------------------------------------------- */
 
-DOH *DohNewFile(DOH *fn, const char *mode) {
+DOH *DohNewFile(DOH *filename, const char *mode, DOHList *newfiles) {
   DohFile *f;
   FILE *file;
-  char *filename;
+  char *filen;
 
-  filename = Char(fn);
-  file = fopen(filename, mode);
+  filen = Char(filename);
+  file = fopen(filen, mode);
   if (!file)
     return 0;
 
@@ -245,6 +248,8 @@ DOH *DohNewFile(DOH *fn, const char *mode) {
     fclose(file);
     return 0;
   }
+  if (newfiles)
+    Append(newfiles, filename);
   f->filep = file;
   f->fd = 0;
   f->closeondel = 1;
