@@ -1,15 +1,17 @@
 /* ----------------------------------------------------------------------------- 
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
+ *
  * list.c
  *
  *     Implements a simple list object.
- * 
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1999-2000.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_list_c[] = "$Id: list.c 9607 2006-12-05 22:11:40Z beazley $";
+char cvsroot_list_c[] = "$Id: list.c 11876 2010-02-27 23:53:33Z wsfulton $";
 
 #include "dohint.h"
 
@@ -345,7 +347,7 @@ DohObjInfo DohListType = {
 
 #define MAXLISTITEMS 8
 
-DOH *DohNewList() {
+DOH *DohNewList(void) {
   List *l;
   int i;
   l = (List *) DohMalloc(sizeof(List));
