@@ -1,6 +1,10 @@
 /* ----------------------------------------------------------------------------- 
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * swigwarn.h
  *
@@ -16,8 +20,6 @@
  * numbers in this file.
  * ----------------------------------------------------------------------------- */
 
-/* $Id: swigwarn.h 10450 2008-05-14 22:12:31Z wsfulton $ */
-
 #ifndef SWIGWARN_H_
 #define SWIGWARN_H_
 
@@ -49,6 +51,7 @@
 #define WARN_DEPRECATED_NOEXTERN      122
 #define WARN_DEPRECATED_NODEFAULT     123
 #define WARN_DEPRECATED_TYPEMAP_LANG  124
+#define WARN_DEPRECATED_INPUT_FILE    125
 
 /* -- Preprocessor -- */
 
@@ -71,7 +74,7 @@
 #define WARN_PARSE_PRIVATE_INHERIT    309
 #define WARN_PARSE_TEMPLATE_REPEAT    310
 #define WARN_PARSE_TEMPLATE_PARTIAL   311
-#define WARN_PARSE_NESTED_CLASS       312
+#define WARN_PARSE_UNNAMED_NESTED_CLASS 312
 #define WARN_PARSE_UNDEFINED_EXTERN   313
 #define WARN_PARSE_KEYWORD            314
 #define WARN_PARSE_USING_UNDEF        315
@@ -83,6 +86,8 @@
 #define WARN_PARSE_BUILTIN_NAME       321
 #define WARN_PARSE_REDUNDANT          322
 #define WARN_PARSE_REC_INHERITANCE    323
+#define WARN_PARSE_NESTED_TEMPLATE    324
+#define WARN_PARSE_NAMED_NESTED_CLASS 325
 
 #define WARN_IGNORE_OPERATOR_NEW        350	/* new */
 #define WARN_IGNORE_OPERATOR_DELETE     351	/* delete */
@@ -247,10 +252,11 @@
 
 /* please leave 850-869 free for Modula 3 */
 
-#define WARN_PHP4_MULTIPLE_INHERITANCE         870
-#define WARN_PHP4_UNKNOWN_PRAGMA               871
+#define WARN_PHP_MULTIPLE_INHERITANCE         870
+#define WARN_PHP_UNKNOWN_PRAGMA               871
+#define WARN_PHP_PUBLIC_BASE                  872
 
-/* please leave 870-889 free for Php */
+/* please leave 870-889 free for PHP */
 
 
 /* Feel free to claim any number in this space that's not currently being used. Just make sure you
