@@ -1,13 +1,17 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * clisp.cxx
  *
  * clisp language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_clisp_cxx[] = "$Id: clisp.cxx 11133 2009-02-20 07:52:24Z wsfulton $";
+char cvsroot_clisp_cxx[] = "$Id: clisp.cxx 11876 2010-02-27 23:53:33Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -25,7 +29,7 @@ public:
   virtual int typedefHandler(Node *n);
   List *entries;
 private:
-  String *get_ffi_type(SwigType *ty);
+  String *get_ffi_type(Node *n, SwigType *ty);
   String *convert_literal(String *num_param, String *type);
   String *strip_parens(String *string);
   int extern_all_flag;
@@ -167,7 +171,7 @@ int CLISP::functionWrapper(Node *n) {
     String *argname = Getattr(p, "name");
     //    SwigType *argtype;
 
-    String *ffitype = get_ffi_type(Getattr(p, "type"));
+    String *ffitype = get_ffi_type(n, Getattr(p, "type"));
 
     int tempargname = 0;
 
@@ -190,7 +194,7 @@ int CLISP::functionWrapper(Node *n) {
   if (ParmList_len(pl) != 0) {
     Printf(f_cl, ")\n");	/* finish arg list */
   }
-  String *ffitype = get_ffi_type(Getattr(n, "type"));
+  String *ffitype = get_ffi_type(n, Getattr(n, "type"));
   if (Strcmp(ffitype, "NIL")) {	//when return type is not nil
     Printf(f_cl, "\t(:return-type %s)\n", ffitype);
   }
@@ -222,7 +226,7 @@ int CLISP::variableWrapper(Node *n) {
     return SWIG_OK;
 
   String *var_name = Getattr(n, "sym:name");
-  String *lisp_type = get_ffi_type(Getattr(n, "type"));
+  String *lisp_type = get_ffi_type(n, Getattr(n, "type"));
   Printf(f_cl, "\n(ffi:def-c-var %s\n (:name \"%s\")\n (:type %s)\n", var_name, var_name, lisp_type);
   Printf(f_cl, "\t(:library +library-name+))\n");
   Append(entries, var_name);
@@ -234,7 +238,7 @@ int CLISP::variableWrapper(Node *n) {
 int CLISP::typedefHandler(Node *n) {
   if (generate_typedef_flag) {
     is_function = 0;
-    Printf(f_cl, "\n(ffi:def-c-type %s %s)\n", Getattr(n, "name"), get_ffi_type(Getattr(n, "type")));
+    Printf(f_cl, "\n(ffi:def-c-type %s %s)\n", Getattr(n, "name"), get_ffi_type(n, Getattr(n, "type")));
   }
 
   return Language::typedefHandler(n);
@@ -290,7 +294,7 @@ int CLISP::classDeclaration(Node *n) {
 
     String *temp = Copy(Getattr(c, "decl"));
     Append(temp, Getattr(c, "type"));	//appending type to the end, otherwise wrong type
-    String *lisp_type = get_ffi_type(temp);
+    String *lisp_type = get_ffi_type(n, temp);
     Delete(temp);
 
     String *slot_name = Getattr(c, "sym:name");
@@ -371,15 +375,20 @@ String *CLISP::convert_literal(String *num_param, String *type) {
   return res;
 }
 
-String *CLISP::get_ffi_type(SwigType *ty) {
-  Hash *typemap = Swig_typemap_search("in", ty, "", 0);
-  if (typemap) {
-    String *typespec = Getattr(typemap, "code");
-    return NewString(typespec);
+String *CLISP::get_ffi_type(Node *n, SwigType *ty) {
+  Node *node = NewHash();
+  Setattr(node, "type", ty);
+  Setfile(node, Getfile(n));
+  Setline(node, Getline(n));
+  const String *tm = Swig_typemap_lookup("in", node, "", 0);
+  Delete(node);
+
+  if (tm) {
+    return NewString(tm);
   } else if (SwigType_ispointer(ty)) {
     SwigType *cp = Copy(ty);
     SwigType_del_pointer(cp);
-    String *inner_type = get_ffi_type(cp);
+    String *inner_type = get_ffi_type(n, cp);
 
     if (SwigType_isfunction(cp)) {
       return inner_type;
@@ -409,12 +418,12 @@ String *CLISP::get_ffi_type(SwigType *ty) {
       Delete(array_dim);
       SwigType_del_array(cp);
       SwigType_add_pointer(cp);
-      String *str = get_ffi_type(cp);
+      String *str = get_ffi_type(n, cp);
       Delete(cp);
       return str;
     } else {
       SwigType_pop_arrays(cp);
-      String *inner_type = get_ffi_type(cp);
+      String *inner_type = get_ffi_type(n, cp);
       Delete(cp);
 
       int ndim = SwigType_array_ndim(ty);
@@ -447,7 +456,7 @@ String *CLISP::get_ffi_type(SwigType *ty) {
     SwigType *cp = Copy(ty);
     SwigType *fn = SwigType_pop_function(cp);
     String *args = NewString("");
-    ParmList *pl = SwigType_function_parms(fn);
+    ParmList *pl = SwigType_function_parms(fn, n);
     if (ParmList_len(pl) != 0) {
       Printf(args, "(:arguments ");
     }
@@ -455,7 +464,7 @@ String *CLISP::get_ffi_type(SwigType *ty) {
     for (Parm *p = pl; p; p = nextSibling(p), argnum++) {
       String *argname = Getattr(p, "name");
       SwigType *argtype = Getattr(p, "type");
-      String *ffitype = get_ffi_type(argtype);
+      String *ffitype = get_ffi_type(n, argtype);
 
       int tempargname = 0;
 
@@ -475,7 +484,7 @@ String *CLISP::get_ffi_type(SwigType *ty) {
     if (ParmList_len(pl) != 0) {
       Printf(args, ")\n");	/* finish arg list */
     }
-    String *ffitype = get_ffi_type(cp);
+    String *ffitype = get_ffi_type(n, cp);
     String *str = NewStringf("(ffi:c-function %s \t\t\t\t(:return-type %s))", args, ffitype);
     Delete(fn);
     Delete(args);
