@@ -1,13 +1,17 @@
 /* ----------------------------------------------------------------------------- 
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * guile.cxx
  *
  * Guile language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_guile_cxx[] = "$Id: guile.cxx 10270 2008-02-27 15:29:55Z wsfulton $";
+char cvsroot_guile_cxx[] = "$Id: guile.cxx 12034 2010-05-21 07:10:12Z olly $";
 
 #include "swigmod.h"
 
@@ -47,6 +51,7 @@ Guile Options (available with -guile)\n\
      -exportprimitive        - Add the (export ...) code from scmstub into the\n\
                                GOOPS file.\n";
 
+static File *f_begin = 0;
 static File *f_runtime = 0;
 static File *f_header = 0;
 static File *f_wrappers = 0;
@@ -174,7 +179,7 @@ public:
 	  }
 	} else if (strcmp(argv[i], "-procdoc") == 0) {
 	  if (argv[i + 1]) {
-	    procdoc = NewFile(argv[i + 1], (char *) "w");
+	    procdoc = NewFile(argv[i + 1], "w", SWIG_output_files());
 	    if (!procdoc) {
 	      FileErrorDisplay(argv[i + 1]);
 	      SWIG_exit(EXIT_FAILURE);
@@ -299,11 +304,12 @@ public:
     /* Initialize all of the output files */
     String *outfile = Getattr(n, "outfile");
 
-    f_runtime = NewFile(outfile, "w");
-    if (!f_runtime) {
+    f_begin = NewFile(outfile, "w", SWIG_output_files());
+    if (!f_begin) {
       FileErrorDisplay(outfile);
       SWIG_exit(EXIT_FAILURE);
     }
+    f_runtime = NewString("");
     f_init = NewString("");
     f_header = NewString("");
     f_wrappers = NewString("");
@@ -311,6 +317,7 @@ public:
     /* Register file targets with the SWIG file handler */
     Swig_register_filebyname("header", f_header);
     Swig_register_filebyname("wrapper", f_wrappers);
+    Swig_register_filebyname("begin", f_begin);
     Swig_register_filebyname("runtime", f_runtime);
     Swig_register_filebyname("init", f_init);
 
@@ -322,10 +329,10 @@ public:
     goopscode = NewString("");
     goopsexport = NewString("");
 
-    Printf(f_runtime, "/* -*- buffer-read-only: t -*- vi: set ro: */\n");
-    Swig_banner(f_runtime);
+    Swig_banner(f_begin);
 
-    Printf(f_runtime, "/* Implementation : GUILE */\n\n");
+    Printf(f_runtime, "\n");
+    Printf(f_runtime, "#define SWIGGUILE\n");
 
     if (!use_scm_interface) {
       if (SwigRuntime == 1)
@@ -358,6 +365,8 @@ public:
       Printf(f_runtime, "\n}\n");
     }
 
+    Printf(f_runtime, "\n");
+
     Language::top(n);
 
     /* Close module */
@@ -392,14 +401,16 @@ public:
     Delete(goopstext);
 
     /* Close all of the files */
-    Dump(f_header, f_runtime);
-    Dump(f_wrappers, f_runtime);
-    Wrapper_pretty_print(f_init, f_runtime);
+    Dump(f_runtime, f_begin);
+    Dump(f_header, f_begin);
+    Dump(f_wrappers, f_begin);
+    Wrapper_pretty_print(f_init, f_begin);
     Delete(f_header);
     Delete(f_wrappers);
     Delete(f_init);
-    Close(f_runtime);
+    Close(f_begin);
     Delete(f_runtime);
+    Delete(f_begin);
     return SWIG_OK;
   }
 
@@ -499,15 +510,15 @@ public:
 				 SWIG_output_directory(),
 				 primitive_name);
       Delete(primitive_name);
-      File *scmstubfile = NewFile(fname, (char *) "w");
+      File *scmstubfile = NewFile(fname, "w", SWIG_output_files());
       if (!scmstubfile) {
 	FileErrorDisplay(fname);
 	SWIG_exit(EXIT_FAILURE);
       }
       Delete(fname);
 
-      Printf(scmstubfile, ";;; -*- buffer-read-only: t -*- vi: set ro: */\n");
-      Printf(scmstubfile, ";;; Automatically generated by SWIG; do not edit.\n\n");
+      Swig_banner_target_lang(scmstubfile, ";;;");
+      Printf(scmstubfile, "\n");
       if (linkage == GUILE_LSTYLE_SIMPLE || linkage == GUILE_LSTYLE_PASSIVE)
 	Printf(scmstubfile, "(define-module (%s))\n\n", mod);
       Delete(mod);
@@ -530,14 +541,14 @@ public:
 
       String *fname = NewStringf("%s%s.scm", SWIG_output_directory(),
 				 module_name);
-      File *goopsfile = NewFile(fname, (char *) "w");
+      File *goopsfile = NewFile(fname, "w", SWIG_output_files());
       if (!goopsfile) {
 	FileErrorDisplay(fname);
 	SWIG_exit(EXIT_FAILURE);
       }
       Delete(fname);
-      Printf(goopsfile, ";;; -*- buffer-read-only: t -*- vi: set ro: */\n");
-      Printf(goopsfile, ";;; Automatically generated by SWIG; do not edit.\n\n");
+      Swig_banner_target_lang(goopsfile, ";;;");
+      Printf(goopsfile, "\n");
       Printf(goopsfile, "(define-module (%s))\n", mod);
       Printf(goopsfile, "%s\n", goopstext);
       Printf(goopsfile, "(use-modules (oop goops) (Swig common))\n");
@@ -646,7 +657,7 @@ public:
     Parm *p;
     String *proc_name = 0;
     char source[256];
-    Wrapper *f = NewWrapper();;
+    Wrapper *f = NewWrapper();
     String *cleanup = NewString("");
     String *outarg = NewString("");
     String *signature = NewString("");
@@ -687,7 +698,7 @@ public:
     Replaceall(proc_name, "_", "-");
 
     /* Emit locals etc. into f->code; figure out which args to ignore */
-    emit_args(d, l, f);
+    emit_parameter_variables(l, f);
 
     /* Attach the standard typemaps */
     emit_attach_parmmaps(l, f);
@@ -702,25 +713,6 @@ public:
     Wrapper_add_local(f, "gswig_result", "SCM gswig_result");
     Wrapper_add_local(f, "gswig_list_p", "SWIGUNUSED int gswig_list_p = 0");
 
-    /* Get the output typemap so we can start generating documentation.  Don't
-       worry, the returned string is saved as 'tmap:out' */
-
-    Swig_typemap_lookup_new("out", n, "result", 0);
-
-    if ((tm = Getattr(n, "tmap:out:doc"))) {
-      Printv(returns, tm, NIL);
-      if (Len(tm) > 0)
-	num_results = 1;
-      else
-	num_results = 0;
-    } else {
-      String *s = SwigType_str(d, 0);
-      Chop(s);
-      Printf(returns, "<%s>", s);
-      Delete(s);
-      num_results = 1;
-    }
-
     /* Open prototype and signature */
 
     Printv(f->def, "static SCM\n", wname, " (", NIL);
@@ -830,6 +822,7 @@ public:
     /* Pass output arguments back to the caller. */
 
     /* Insert argument output code */
+    String *returns_argout = NewString("");
     for (p = l; p;) {
       if ((tm = Getattr(p, "tmap:argout"))) {
 	Replaceall(tm, "$source", Getattr(p, "lname"));
@@ -838,7 +831,7 @@ public:
 	Replaceall(tm, "$input", Getattr(p, "emit:input"));
 	Printv(outarg, tm, "\n", NIL);
 	if (procdoc) {
-	  if (handle_documentation_typemap(returns, ", ", p, "tmap:argout:doc", "$NAME (of type $type)")) {
+	  if (handle_documentation_typemap(returns_argout, ", ", p, "tmap:argout:doc", "$NAME (of type $type)")) {
 	    /* A documentation typemap that is not the empty string
 	       indicates that a value is returned to Scheme. */
 	    num_results++;
@@ -881,13 +874,14 @@ public:
     // Now write code to make the function call
     if (!use_scm_interface)
       Printv(f->code, tab4, "gh_defer_ints();\n", NIL);
-    emit_action(n, f);
+
+    String *actioncode = emit_action(n);
+
     if (!use_scm_interface)
-      Printv(f->code, tab4, "gh_allow_ints();\n", NIL);
+      Printv(actioncode, tab4, "gh_allow_ints();\n", NIL);
 
     // Now have return value, figure out what to do with it.
-
-    if ((tm = Getattr(n, "tmap:out"))) {
+    if ((tm = Swig_typemap_lookup_out("out", n, "result", f, actioncode))) {
       Replaceall(tm, "$result", "gswig_result");
       Replaceall(tm, "$target", "gswig_result");
       Replaceall(tm, "$source", "result");
@@ -899,6 +893,24 @@ public:
     } else {
       throw_unhandled_guile_type_error(d);
     }
+    emit_return_variable(n, d, f);
+
+    // Documentation
+    if ((tm = Getattr(n, "tmap:out:doc"))) {
+      Printv(returns, tm, NIL);
+      if (Len(tm) > 0)
+	num_results = 1;
+      else
+	num_results = 0;
+    } else {
+      String *s = SwigType_str(d, 0);
+      Chop(s);
+      Printf(returns, "<%s>", s);
+      Delete(s);
+      num_results = 1;
+    }
+    Append(returns, returns_argout);
+
 
     // Dump the argument output code
     Printv(f->code, outarg, NIL);
@@ -909,13 +921,13 @@ public:
     // Look for any remaining cleanup
 
     if (GetFlag(n, "feature:new")) {
-      if ((tm = Swig_typemap_lookup_new("newfree", n, "result", 0))) {
+      if ((tm = Swig_typemap_lookup("newfree", n, "result", 0))) {
 	Replaceall(tm, "$source", "result");
 	Printv(f->code, tm, "\n", NIL);
       }
     }
     // Free any memory allocated by the function being wrapped..
-    if ((tm = Swig_typemap_lookup_new("ret", n, "result", 0))) {
+    if ((tm = Swig_typemap_lookup("ret", n, "result", 0))) {
       Replaceall(tm, "$source", "result");
       Printv(f->code, tm, "\n", NIL);
     }
@@ -1123,6 +1135,7 @@ public:
     Delete(method_signature);
     Delete(primitive_args);
     Delete(doc_body);
+    Delete(returns_argout);
     Delete(returns);
     Delete(tmp);
     Delete(scheme_arg_names);
@@ -1177,12 +1190,12 @@ public:
       if (!GetFlag(n, "feature:immutable")) {
 	/* Check for a setting of the variable value */
 	Printf(f->code, "if (s_0 != SCM_UNDEFINED) {\n");
-	if ((tm = Swig_typemap_lookup_new("varin", n, name, 0))) {
+	if ((tm = Swig_typemap_lookup("varin", n, name, 0))) {
 	  Replaceall(tm, "$source", "s_0");
 	  Replaceall(tm, "$input", "s_0");
 	  Replaceall(tm, "$target", name);
 	  /* Printv(f->code,tm,"\n",NIL); */
-	  emit_action_code(n, f, tm);
+	  emit_action_code(n, f->code, tm);
 	} else {
 	  throw_unhandled_guile_type_error(t);
 	}
@@ -1191,12 +1204,12 @@ public:
       // Now return the value of the variable (regardless
       // of evaluating or setting)
 
-      if ((tm = Swig_typemap_lookup_new("varout", n, name, 0))) {
+      if ((tm = Swig_typemap_lookup("varout", n, name, 0))) {
 	Replaceall(tm, "$source", name);
 	Replaceall(tm, "$target", "gswig_result");
 	Replaceall(tm, "$result", "gswig_result");
 	/* Printv(f->code,tm,"\n",NIL); */
-	emit_action_code(n, f, tm);
+	emit_action_code(n, f->code, tm);
       } else {
 	throw_unhandled_guile_type_error(t);
       }
@@ -1387,7 +1400,7 @@ public:
       rvalue = NewString(value);
     }
 
-    if ((tm = Swig_typemap_lookup_new("constant", n, name, 0))) {
+    if ((tm = Swig_typemap_lookup("constant", n, name, 0))) {
       Replaceall(tm, "$source", rvalue);
       Replaceall(tm, "$value", rvalue);
       Replaceall(tm, "$target", name);
@@ -1398,16 +1411,18 @@ public:
     }
     {
       /* Hack alert: will cleanup later -- Dave */
-      Node *n = NewHash();
-      Setattr(n, "name", var_name);
-      Setattr(n, "sym:name", iname);
-      Setattr(n, "type", nctype);
-      SetFlag(n, "feature:immutable");
+      Node *nn = NewHash();
+      Setfile(nn, Getfile(n));
+      Setline(nn, Getline(n));
+      Setattr(nn, "name", var_name);
+      Setattr(nn, "sym:name", iname);
+      Setattr(nn, "type", nctype);
+      SetFlag(nn, "feature:immutable");
       if (constasvar) {
-	SetFlag(n, "feature:constasvar");
+	SetFlag(nn, "feature:constasvar");
       }
-      variableWrapper(n);
-      Delete(n);
+      variableWrapper(nn);
+      Delete(nn);
     }
     Delete(var_name);
     Delete(nctype);
@@ -1647,11 +1662,11 @@ public:
   /* ------------------------------------------------------------
    * goopsNameMapping()
    * Maps the identifier from C++ to the GOOPS based * on command 
-   * line paramaters and such.
+   * line parameters and such.
    * If class_name = "" that means the mapping is for a function or
    * variable not attached to any class.
    * ------------------------------------------------------------ */
-  String *goopsNameMapping(String *name, String_or_char *class_name) {
+  String *goopsNameMapping(String *name, const_String_or_char_ptr class_name) {
     String *n = NewString("");
 
     if (Strcmp(class_name, "") == 0) {
