@@ -1,13 +1,17 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * r.cxx
  *
  * R language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_r_cxx[] = "$Id: r.cxx 11454 2009-07-26 21:21:26Z wsfulton $";
+char cvsroot_r_cxx[] = "$Id: r.cxx 12023 2010-05-12 04:12:42Z drjoe $";
 
 #include "swigmod.h"
 
@@ -66,40 +70,6 @@ static String * getRTypeName(SwigType *t, int *outCount = NULL) {
   */
 }
 
-#if 0
-static String * getRType(Node *n) {
-  SwigType *elType = Getattr(n, "type");
-  SwigType *elDecl = Getattr(n, "decl");
-  //XXX How can we tell if this is already done.
-  SwigType_push(elType, elDecl);
-  String *ans;
-
-  String *rtype = Swig_typemap_lookup("rtype", n, "", 0);
-  String *i = getRTypeName(elType);
-
-  if(Len(i) == 0) {
-    SwigType *td = SwigType_typedef_resolve(elType);
-    if(td) {
-      //     Printf(stderr, "Resolving typedef %s -> %s\n", elType, td);
-      i = getRTypeName(td);
-    }
-  }
-  //  Printf(stderr, "<getRType> i = %s,  rtype = %s  (for %s)\n", 
-  //	 i, rtype, elType);
-  if(rtype) {
-    ans = NewString("");
-    Printf(ans, "%s", rtype);
-    Replaceall(ans, "$R_class", Char(i));
-    //	Printf(stderr, "Found r type in typemap for %s (for %s) => %s (%s) => %s\n", 
-    //         SwigType_str(elType, 0), Getattr(n, "name"), rtype, i, ans);
-  } else {
-    ans = i;
-  }
-  
-  return(ans);
-}
-#endif
-
 /*********************
  Tries to get the name of the R class corresponding  to the given type
   e.g. struct A * is ARef,  struct A**  is  ARefRef.
@@ -530,12 +500,7 @@ int R::getFunctionPointerNumArgs(Node *n, SwigType *tt) {
   n = Getattr(n, "type");
   if (debugMode)
     Printf(stderr, "type: %s\n", n);
-#if 0
-  SwigType *tmp = SwigType_typedef_resolve(tt);
-  
-  n = SwigType_typedef_resolve(tt);
-#endif
-  
+
   ParmList *parms = Getattr(n, "parms");
   if (debugMode)
     Printf(stderr, "parms = %p\n", parms);
@@ -588,13 +553,13 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
 
   //   ParmList *parms = Getattr(n, "parms");
   // memory leak
-  ParmList *parms = SwigType_function_parms(SwigType_del_pointer(Copy(t)));
+  ParmList *parms = SwigType_function_parms(SwigType_del_pointer(Copy(t)), n);
 
 
-  //  if (debugMode) {
+  if (debugMode) {
     Printf(stderr, "Type: %s\n", t);
     Printf(stderr, "Return type: %s\n", SwigType_base(t));
-    //}
+  }
   
   bool isVoidType = Strcmp(rettype, "void") == 0;
   if (debugMode)
@@ -712,10 +677,7 @@ String * R::createFunctionPointerHandler(SwigType *t, Node *n, int *numArgs) {
        XXX  Have to be a little more clever so that we can deal with struct A * - the * is getting lost.
        Is this still true? If so, will a SwigType_push() solve things?
     */
-    Node *bbase = NewHash();
-    
-    Setattr(bbase, "type", rettype);
-    Setattr(bbase, "name", NewString("result"));
+    Parm *bbase = NewParm(rettype, "result", n);
     String *returnTM = Swig_typemap_lookup("in", bbase, "result", f);
     if(returnTM) {
       String *tm = returnTM;
@@ -1101,7 +1063,7 @@ int R::OutputMemberReferenceMethod(String *className, int isSet,
     Delete(pitem);
   }
   Delete(itemList);
-  Printf(f->code, ")\n");
+  Printf(f->code, ");\n");
   
   if (!isSet && varaccessor > 0) {
     Printf(f->code, "%svaccessors = c(", tab8);
@@ -1117,7 +1079,7 @@ int R::OutputMemberReferenceMethod(String *className, int isSet,
 	Printf(f->code, "'%s'%s", item, vcount < varaccessor ? ", " : "");
       }
     }
-    Printf(f->code, ")\n");
+    Printf(f->code, ");\n");
   }
   
   
@@ -1129,24 +1091,24 @@ int R::OutputMemberReferenceMethod(String *className, int isSet,
 	"stop(\"No ", (isSet ? "modifiable" : "accessible"), " field named \", name, \" in ", className,
 	": fields are \", paste(names(accessorFuns), sep = \", \")", 
 	")", "\n}\n", NIL); */
-  Printv(f->code, tab8,
-	 "idx = pmatch(name, names(accessorFuns))\n",
+  Printv(f->code, ";", tab8,
+	 "idx = pmatch(name, names(accessorFuns));\n",
 	 tab8,
 	 "if(is.na(idx)) \n",
 	 tab8, tab4, NIL);
-  Printf(f->code, "return(callNextMethod(x, name%s))\n",
+  Printf(f->code, "return(callNextMethod(x, name%s));\n",
 	 isSet ? ", value" : "");
-  Printv(f->code, tab8, "f = accessorFuns[[idx]]\n", NIL);
+  Printv(f->code, tab8, "f = accessorFuns[[idx]];\n", NIL);
   if(isSet) {
-    Printv(f->code, tab8, "f(x, value)\n", NIL);
-    Printv(f->code, tab8, "x\n", NIL); // make certain to return the S value.
+    Printv(f->code, tab8, "f(x, value);\n", NIL);
+    Printv(f->code, tab8, "x;\n", NIL); // make certain to return the S value.
   } else {
-    Printv(f->code, tab8, "formals(f)[[1]] = x\n", NIL);
+    Printv(f->code, tab8, "formals(f)[[1]] = x;\n", NIL);
     if (varaccessor) {
       Printv(f->code, tab8,
-	     "if (is.na(match(name, vaccessors))) f else f(x)\n", NIL);
+	     "if (is.na(match(name, vaccessors))) f else f(x);\n", NIL);
     } else {
-      Printv(f->code, tab8, "f\n", NIL);
+      Printv(f->code, tab8, "f;\n", NIL);
     }
   }
   Printf(f->code, "}\n");
@@ -1157,15 +1119,15 @@ int R::OutputMemberReferenceMethod(String *className, int isSet,
 	 isSet ? "<-" : "", 
 	 getRClassName(className)); 
   Wrapper_print(f, out);
-  Printf(out, ")\n");
+  Printf(out, ");\n");
   
   if(isSet) {
     Printf(out, "setMethod('[[<-', c('_p%s', 'character'),", 
 	   getRClassName(className)); 
-    Insert(f->code, 2, "name = i\n");
+    Insert(f->code, 2, "name = i;\n");
     Printf(attr->code, "%s", f->code);
     Wrapper_print(attr, out);
-    Printf(out, ")\n");
+    Printf(out, ");\n");
   }
   
   DelWrapper(attr);
@@ -1489,8 +1451,8 @@ static List * Swig_overload_rank(Node *n,
 	  }
 	  if (!differ) {
 	    /* See if declarations differ by const only */
-	    String *d1 = Getattr(nodes[i].n,"decl");
-	    String *d2 = Getattr(nodes[j].n,"decl");
+	    String *d1 = Getattr(nodes[i].n, "decl");
+	    String *d2 = Getattr(nodes[j].n, "decl");
 	    if (d1 && d2) {
 	      String *dq1 = Copy(d1);
 	      String *dq2 = Copy(d2);
@@ -1500,49 +1462,47 @@ static List * Swig_overload_rank(Node *n,
 	      if (SwigType_isconst(d2)) {
 		Delete(SwigType_pop(dq2));
 	      }
-	      if (Strcmp(dq1,dq2) == 0) {
-		
+	      if (Strcmp(dq1, dq2) == 0) {
+
 		if (SwigType_isconst(d1) && !SwigType_isconst(d2)) {
-                  if (script_lang_wrapping) {
-                    // Swap nodes so that the const method gets ignored (shadowed by the non-const method)
-                    Overloaded t = nodes[i];
-                    nodes[i] = nodes[j];
-                    nodes[j] = t;
-                  }
+		  if (script_lang_wrapping) {
+		    // Swap nodes so that the const method gets ignored (shadowed by the non-const method)
+		    Overloaded t = nodes[i];
+		    nodes[i] = nodes[j];
+		    nodes[j] = t;
+		  }
 		  differ = 1;
 		  if (!nodes[j].error) {
-                    if (script_lang_wrapping) {
+		    if (script_lang_wrapping) {
 		      Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[j].n), Getline(nodes[j].n),
-				   "Overloaded %s(%s) const ignored. Non-const method at %s:%d used.\n",
-				   Getattr(nodes[j].n,"name"), ParmList_errorstr(nodes[j].parms),
-				   Getfile(nodes[i].n), Getline(nodes[i].n));
-                    } else {
-                      if (!Getattr(nodes[j].n, "overload:ignore"))
-		        Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
-				     "Overloaded method %s(%s) ignored. Method %s(%s) const at %s:%d used.\n",
-				     Getattr(nodes[j].n,"name"), ParmList_errorstr(nodes[j].parms),
-			             Getattr(nodes[i].n,"name"), ParmList_errorstr(nodes[i].parms),
-				     Getfile(nodes[i].n), Getline(nodes[i].n));
-                    }
+				   "Overloaded method %s ignored,\n", Swig_name_decl(nodes[j].n));
+		      Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[i].n), Getline(nodes[i].n),
+				   "using non-const method %s instead.\n", Swig_name_decl(nodes[i].n));
+		    } else {
+		      if (!Getattr(nodes[j].n, "overload:ignore"))
+			Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
+				     "Overloaded method %s ignored,\n", Swig_name_decl(nodes[j].n));
+			Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[i].n), Getline(nodes[i].n),
+				     "using %s instead.\n", Swig_name_decl(nodes[i].n));
+		    }
 		  }
 		  nodes[j].error = 1;
 		} else if (!SwigType_isconst(d1) && SwigType_isconst(d2)) {
 		  differ = 1;
 		  if (!nodes[j].error) {
-                    if (script_lang_wrapping) {
+		    if (script_lang_wrapping) {
 		      Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[j].n), Getline(nodes[j].n),
-				   "Overloaded %s(%s) const ignored. Non-const method at %s:%d used.\n",
-				   Getattr(nodes[j].n,"name"), ParmList_errorstr(nodes[j].parms),
-				   Getfile(nodes[i].n), Getline(nodes[i].n));
-                    } else {
-                      if (!Getattr(nodes[j].n, "overload:ignore"))
-		        Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
-				     "Overloaded method %s(%s) const ignored. Method %s(%s) at %s:%d used.\n",
-				     Getattr(nodes[j].n,"name"), ParmList_errorstr(nodes[j].parms),
-			             Getattr(nodes[i].n,"name"), ParmList_errorstr(nodes[i].parms),
-				     Getfile(nodes[i].n), Getline(nodes[i].n));
-                    }
-                  }
+				   "Overloaded method %s ignored,\n", Swig_name_decl(nodes[j].n));
+		      Swig_warning(WARN_LANG_OVERLOAD_CONST, Getfile(nodes[i].n), Getline(nodes[i].n),
+				   "using non-const method %s instead.\n", Swig_name_decl(nodes[i].n));
+		    } else {
+		      if (!Getattr(nodes[j].n, "overload:ignore"))
+			Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
+				     "Overloaded method %s ignored,\n", Swig_name_decl(nodes[j].n));
+			Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[i].n), Getline(nodes[i].n),
+				     "using %s instead.\n", Swig_name_decl(nodes[i].n));
+		    }
+		  }
 		  nodes[j].error = 1;
 		}
 	      }
@@ -1552,18 +1512,18 @@ static List * Swig_overload_rank(Node *n,
 	  }
 	  if (!differ) {
 	    if (!nodes[j].error) {
-              if (script_lang_wrapping) {
-	        Swig_warning(WARN_LANG_OVERLOAD_SHADOW, Getfile(nodes[j].n), Getline(nodes[j].n),
-			     "Overloaded method %s is shadowed by %s at %s:%d.\n",
-			     Swig_name_decl(nodes[j].n), Swig_name_decl(nodes[i].n),
-			     Getfile(nodes[i].n), Getline(nodes[i].n));
-              } else {
-                if (!Getattr(nodes[j].n, "overload:ignore"))
-	          Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
-			       "Overloaded method %s ignored. Method %s at %s:%d used.\n",
-			       Swig_name_decl(nodes[j].n), Swig_name_decl(nodes[i].n),
-			       Getfile(nodes[i].n), Getline(nodes[i].n));
-              }
+	      if (script_lang_wrapping) {
+		Swig_warning(WARN_LANG_OVERLOAD_SHADOW, Getfile(nodes[j].n), Getline(nodes[j].n),
+			     "Overloaded method %s effectively ignored,\n", Swig_name_decl(nodes[j].n));
+		Swig_warning(WARN_LANG_OVERLOAD_SHADOW, Getfile(nodes[i].n), Getline(nodes[i].n),
+			     "as it is shadowed by %s.\n", Swig_name_decl(nodes[i].n));
+	      } else {
+		if (!Getattr(nodes[j].n, "overload:ignore"))
+		  Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[j].n), Getline(nodes[j].n),
+			       "Overloaded method %s ignored,\n", Swig_name_decl(nodes[j].n));
+		  Swig_warning(WARN_LANG_OVERLOAD_IGNORED, Getfile(nodes[i].n), Getline(nodes[i].n),
+			       "using %s instead.\n", Swig_name_decl(nodes[i].n));
+	      }
 	      nodes[j].error = 1;
 	    }
 	  }
@@ -1601,9 +1561,9 @@ void R::dispatchFunction(Node *n) {
   List *dispatch = Swig_overload_rank(n, true);
   int   nfunc = Len(dispatch);
   Printv(f->code, 
-	 "argtypes <- mapply(class, list(...))\n",
-	 "argv <- list(...)\n",
-	 "argc <- length(argtypes)\n", NIL );
+	 "argtypes <- mapply(class, list(...));\n",
+	 "argv <- list(...);\n",
+	 "argc <- length(argtypes);\n", NIL );
 
   Printf(f->code, "# dispatch functions %d\n", nfunc);
   int cur_args = -1;
@@ -1649,16 +1609,16 @@ void R::dispatchFunction(Node *n) {
 	}
 	p = Getattr(p, "tmap:in:next");
       }
-      Printf(f->code, ") { f <- %s%s }\n", sfname, overname);
+      Printf(f->code, ") { f <- %s%s; }\n", sfname, overname);
     } else {
-      Printf(f->code, "f <- %s%s", sfname, overname);
+      Printf(f->code, "f <- %s%s; ", sfname, overname);
     }
   }
   if (cur_args != -1) {
     Printv(f->code, "}", NIL);
   }
-  Printv(f->code, "\nf(...)", NIL);
-  Printv(f->code, "\n}", NIL);
+  Printv(f->code, ";\nf(...)", NIL);
+  Printv(f->code, ";\n}", NIL);
   Wrapper_print(f, sfile);
   Printv(sfile, "# Dispatch function\n", NIL);
   DelWrapper(f);
@@ -1713,7 +1673,7 @@ int R::functionWrapper(Node *n) {
     }
     p = nextSibling(p);
   } 
-  
+
   String *unresolved_return_type = 
     Copy(type);
   if (expandTypedef(type) &&
@@ -1771,7 +1731,7 @@ int R::functionWrapper(Node *n) {
 
   Wrapper *f = NewWrapper();
   Wrapper *sfun = NewWrapper();
-    
+  
   int isVoidReturnType = (Strcmp(type, "void") == 0);
   // Need to use the unresolved return type since 
   // typedef resolution removes the const which causes a 
@@ -1886,16 +1846,16 @@ int R::functionWrapper(Node *n) {
 	String *snargs = NewStringf("%d", nargs);
 	Printv(sfun->code, "if(is.function(", name, ")) {", "\n",
 	       "assert('...' %in% names(formals(", name, 
-	       ")) || length(formals(", name, ")) >= ", snargs, ")\n} ", NIL);
+	       ")) || length(formals(", name, ")) >= ", snargs, ");\n} ", NIL);
 	Delete(snargs);
 
 	Printv(sfun->code, "else {\n",
 	       "if(is.character(", name, ")) {\n",
-	       name, " = getNativeSymbolInfo(", name, ")",
-	       "\n}\n",
+	       name, " = getNativeSymbolInfo(", name, ");",
+	       "\n};\n",
 	       "if(is(", name, ", \"NativeSymbolInfo\")) {\n",
-	       name, " = ", name, "$address", "\n}\n",
-	       "}\n",
+	       name, " = ", name, "$address", ";\n}\n",
+	       "}; \n",
 	       NIL);
       } else {
 	Printf(sfun->code, "%s\n", tm);
@@ -2076,7 +2036,13 @@ int R::functionWrapper(Node *n) {
   Printv(f->code, cleanup, NIL);
   Delete(cleanup);
 
-
+  /* Look to see if there is any newfree cleanup code */
+  if (GetFlag(n, "feature:new")) {
+    if ((tm = Swig_typemap_lookup("newfree", n, "result", 0))) {
+      Replaceall(tm, "$source", "result");	/* deprecated */
+      Printf(f->code, "%s\n", tm);
+    }
+  }
 
   Printv(f->code, UnProtectWrapupCode, NIL);
 
@@ -2090,10 +2056,20 @@ int R::functionWrapper(Node *n) {
   }
 
 
-  Printv(sfun->code, (Len(tm) ? "ans = " : ""), ".Call('", wname, 
-	 "', ", sargs, "PACKAGE='", Rpackage, "')\n", NIL);
+  Printv(sfun->code, ";", (Len(tm) ? "ans = " : ""), ".Call('", wname, 
+	 "', ", sargs, "PACKAGE='", Rpackage, "');\n", NIL);
   if(Len(tm))
-    Printf(sfun->code, "%s\n\nans\n", tm);
+    {
+      Printf(sfun->code, "%s\n\n", tm); 
+      if (constructor)
+	{ 
+	  String *finalizer = NewString(iname);
+	  Replace(finalizer, "new_", "", DOH_REPLACE_FIRST);
+	  Printf(sfun->code, "reg.finalizer(ans, delete_%s)\n", finalizer);
+	}                                                                      
+      Printf(sfun->code, "ans\n");
+    }
+
   if (destructor)
     Printv(f->code, "R_ClearExternalPtr(self);\n", NIL);
 
@@ -2452,11 +2428,11 @@ int R::generateCopyRoutines(Node *n) {
 
     /* The S functions to get and set the member value. */
     String *elNameT = replaceInitialDash(elName);
-    Printf(copyToR->code, "obj@%s = value$%s\n", elNameT, elNameT);
-    Printf(copyToC->code, "obj$%s = value@%s\n", elNameT, elNameT);
+    Printf(copyToR->code, "obj@%s = value$%s;\n", elNameT, elNameT);
+    Printf(copyToC->code, "obj$%s = value@%s;\n", elNameT, elNameT);
     Delete(elNameT);
   }
-  Printf(copyToR->code, "obj\n}\n\n");
+  Printf(copyToR->code, "obj;\n}\n\n");
   String *rclassName = getRClassNameCopyStruct(type, 0); // without the Ref.
   Printf(sfile, "# Start definition of copy functions & methods for %s\n", rclassName);  
   
@@ -2466,9 +2442,9 @@ int R::generateCopyRoutines(Node *n) {
   
   
   Printf(sfile, "# Start definition of copy methods for %s\n", rclassName);  
-  Printf(sfile, "setMethod('copyToR', '_p_%s', CopyToR%s)\n", rclassName, 
+  Printf(sfile, "setMethod('copyToR', '_p_%s', CopyToR%s);\n", rclassName, 
 	 mangledName);
-  Printf(sfile, "setMethod('copyToC', '%s', CopyToC%s)\n\n", rclassName, 
+  Printf(sfile, "setMethod('copyToC', '%s', CopyToC%s);\n\n", rclassName, 
 	 mangledName);
   
   Printf(sfile, "# End definition of copy methods for %s\n", rclassName);  
