@@ -1,6 +1,10 @@
 /* ----------------------------------------------------------------------------- 
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * typepass.cxx
  *
@@ -12,7 +16,7 @@
  * and other information needed for compilation.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typepass_cxx[] = "$Id: typepass.cxx 10003 2007-10-17 21:42:11Z wsfulton $";
+char cvsroot_typepass_cxx[] = "$Id: typepass.cxx 12036 2010-05-22 22:49:47Z wsfulton $";
 
 #include "swigmod.h"
 #include "cparse.h"
@@ -32,6 +36,7 @@ class TypePass:private Dispatcher {
   Node *module;
   int importmode;
   String *nsname;
+  String *nssymname;
   Hash *classhash;
   List *normalize;
 
@@ -205,7 +210,7 @@ class TypePass:private Dispatcher {
 		Setmeta(bname, "already_warned", "1");
 	      }
 	    }
-	    SwigType_inherit(clsname, bname, cast);
+	    SwigType_inherit(clsname, bname, cast, 0);
 	  }
 	}
       }
@@ -225,7 +230,38 @@ class TypePass:private Dispatcher {
       String *bname = Getattr(n, "name");
       Node *bclass = n;		/* Getattr(n,"class"); */
       Hash *scopes = Getattr(bclass, "typescope");
-      SwigType_inherit(clsname, bname, cast);
+      SwigType_inherit(clsname, bname, cast, 0);
+      String *smartptr = Getattr(first, "feature:smartptr");
+      if (smartptr) {
+	SwigType *smart = 0;
+	SwigType *spt = Swig_cparse_type(smartptr);
+	if (spt) {
+	  smart = SwigType_typedef_resolve_all(spt);
+	  Delete(spt);
+	  /* Record a (fake) inheritance relationship between smart pointer
+	     and smart pointer to base class, so that smart pointer upcasts
+	     are automatically generated. */
+          SwigType *bsmart = Copy(smart);
+          SwigType *rclsname = SwigType_typedef_resolve_all(clsname);
+          SwigType *rbname = SwigType_typedef_resolve_all(bname);
+	  Replaceall(bsmart, rclsname, rbname);
+          Delete(rclsname);
+          Delete(rbname);
+	  String *smartnamestr = SwigType_namestr(smart);
+	  String *bsmartnamestr = SwigType_namestr(bsmart);
+	  /* construct casting code */
+	  String *convcode = NewStringf("\n    *newmemory = SWIG_CAST_NEW_MEMORY;\n    return (void *) new %s(*(%s *)$from);\n", bsmartnamestr, smartnamestr);
+	  Delete(bsmartnamestr);
+	  Delete(smartnamestr);
+	  /* setup inheritance relationship between smart pointer templates */
+	  SwigType_inherit(smart, bsmart, 0, convcode);
+	  Delete(convcode);
+	  Delete(bsmart);
+	  Delete(smart);
+	} else {
+	  Swig_error(Getfile(first), Getline(first), "Invalid type (%s) in 'smartptr' feature for class %s.\n", smartptr, clsname);
+	}
+      }
       if (!importmode) {
 	String *btype = Copy(bname);
 	SwigType_add_pointer(btype);
@@ -292,6 +328,7 @@ class TypePass:private Dispatcher {
     inclass = 0;
     normalize = 0;
     nsname = 0;
+    nssymname = 0;
     classhash = Getattr(n, "classes");
     emit_children(n);
     normalize_list();
@@ -411,6 +448,10 @@ class TypePass:private Dispatcher {
 	Setattr(n, "tdname", tdname);
       }
     }
+    if (nssymname) {
+      if (GetFlag(n, "feature:nspace"))
+	Setattr(n, "sym:nspace", nssymname);
+    }
     SwigType_new_scope(scopename);
     SwigType_attach_symtab(Getattr(n, "symtab"));
 
@@ -531,7 +572,9 @@ class TypePass:private Dispatcher {
 	}
       }
       String *oldnsname = nsname;
+      String *oldnssymname = nssymname;
       nsname = Swig_symbol_qualified(Getattr(n, "symtab"));
+      nssymname = Swig_symbol_qualified_language_scopename(Getattr(n, "symtab"));
       symtab = Swig_symbol_setscope(Getattr(n, "symtab"));
       emit_children(n);
       Swig_symbol_setscope(symtab);
@@ -553,6 +596,8 @@ class TypePass:private Dispatcher {
       }
       normalize = olist;
 
+      Delete(nssymname);
+      nssymname = oldnssymname;
       Delete(nsname);
       nsname = oldnsname;
       return SWIG_OK;
@@ -564,7 +609,6 @@ class TypePass:private Dispatcher {
    * ------------------------------------------------------------ */
 
   virtual int cDeclaration(Node *n) {
-
     if (NoExcept) {
       Delattr(n, "throws");
     }
@@ -731,6 +775,54 @@ class TypePass:private Dispatcher {
     }
     Setattr(n, "enumtype", enumtype);
 
+    if (nssymname) {
+      if (GetFlag(n, "feature:nspace"))
+	Setattr(n, "sym:nspace", nssymname);
+    }
+
+    // This block of code is for dealing with %ignore on an enum item where the target language
+    // attempts to use the C enum value in the target language itself and expects the previous enum value
+    // to be one more than the previous value... the previous enum item might not exist if it is ignored!
+    // - It sets the first non-ignored enum item with the "firstenumitem" attribute.
+    // - It adds an enumvalue attribute if the previous enum item is ignored
+    {
+      Node *c;
+      int count = 0;
+      String *previous = 0;
+      bool previous_ignored = false;
+      bool firstenumitem = false;
+      for (c = firstChild(n); c; c = nextSibling(c)) {
+	assert(strcmp(Char(nodeType(c)), "enumitem") == 0);
+
+	bool reset;
+	String *enumvalue = Getattr(c, "enumvalue");
+	if (GetFlag(c, "feature:ignore")) {
+	  reset = enumvalue ? true : false;
+	  previous_ignored = true;
+	} else {
+	  if (!enumvalue && previous_ignored) {
+	    if (previous)
+	      Setattr(c, "enumvalue", NewStringf("(%s) + %d", previous, count+1));
+	    else
+	      Setattr(c, "enumvalue", NewStringf("%d", count));
+	    SetFlag(c, "virtenumvalue"); // identify enumvalue as virtual, ie not from the parsed source
+	  }
+	  if (!firstenumitem) {
+	    SetFlag(c, "firstenumitem");
+	    firstenumitem = true;
+	  }
+	  reset = true;
+	  previous_ignored = false;
+	}
+	if (reset) {
+	  previous = enumvalue ? enumvalue : Getattr(c, "name");
+	  count = 0;
+	} else {
+	  count++;
+	}
+      }
+    }
+
     emit_children(n);
     return SWIG_OK;
   }
@@ -746,21 +838,28 @@ class TypePass:private Dispatcher {
       value = name;
     if (Strcmp(value, name) == 0) {
       String *new_value;
-      if (((nsname) || (inclass)) && cparse_cplusplus) {
+      if ((nsname || inclass) && cparse_cplusplus) {
 	new_value = NewStringf("%s::%s", SwigType_namestr(Swig_symbol_qualified(n)), value);
       } else {
 	new_value = NewString(value);
       }
+      if ((nsname || inclass) && !cparse_cplusplus) {
+	String *cppvalue = NewStringf("%s::%s", SwigType_namestr(Swig_symbol_qualified(n)), value);
+	Setattr(n, "cppvalue", cppvalue); /* for target languages that always generate C++ code even when wrapping C code */
+      }
       Setattr(n, "value", new_value);
       Delete(new_value);
     }
-    // Make up an enumvalue if one was not specified in the parsed code
-    if (Getattr(n, "_last") && !Getattr(n, "enumvalue")) {	// Only the first enum item has _last set
-      Setattr(n, "enumvalueex", "0");
-    }
     Node *next = nextSibling(n);
-    if (next && !Getattr(next, "enumvalue")) {
-      Setattr(next, "enumvalueex", NewStringf("%s + 1", Getattr(n, "sym:name")));
+
+    // Make up an enumvalue if one was not specified in the parsed code (not designed to be used on enum items and %ignore - enumvalue will be set instead)
+    if (!GetFlag(n, "feature:ignore")) {
+      if (Getattr(n, "_last") && !Getattr(n, "enumvalue")) {	// Only the first enum item has _last set (Note: first non-ignored enum item has firstenumitem set)
+	Setattr(n, "enumvalueex", "0");
+      }
+      if (next && !Getattr(next, "enumvalue")) {
+	Setattr(next, "enumvalueex", NewStringf("%s + 1", Getattr(n, "sym:name")));
+      }
     }
 
     return SWIG_OK;
@@ -777,6 +876,35 @@ class TypePass:private Dispatcher {
     return enumDeclaration(n);
   }
 
+#ifdef DEBUG_OVERLOADED
+  static void show_overloaded(Node *n) {
+    Node *c = Getattr(n, "sym:overloaded");
+    Node *checkoverloaded = c;
+    Printf(stdout, "-------------------- overloaded start %s sym:overloaded():%p -------------------------------\n", Getattr(n, "name"), c);
+    while (c) {
+      if (Getattr(c, "error")) {
+        c = Getattr(c, "sym:nextSibling");
+        continue;
+      }
+      if (Getattr(c, "sym:overloaded") != checkoverloaded) {
+        Printf(stdout, "sym:overloaded error c:%p checkoverloaded:%p\n", c, checkoverloaded);
+        Swig_print_node(c);
+        exit (1);
+      }
+
+      String *decl = Strcmp(nodeType(c), "using") == 0 ? NewString("------") : Getattr(c, "decl");
+      Printf(stdout, "  show_overloaded %s::%s(%s)          [%s] nodeType:%s\n", parentNode(c) ? Getattr(parentNode(c), "name") : "NOPARENT", Getattr(c, "name"), decl, Getattr(c, "sym:overname"), nodeType(c));
+      if (!Getattr(c, "sym:overloaded")) {
+        Printf(stdout, "sym:overloaded error.....%p\n", c);
+        Swig_print_node(c);
+        exit (1);
+      }
+      c = Getattr(c, "sym:nextSibling");
+    }
+    Printf(stdout, "-------------------- overloaded end   %s -------------------------------\n", Getattr(n, "name"));
+  }
+#endif
+
   /* ------------------------------------------------------------
    * usingDeclaration()
    * ------------------------------------------------------------ */
@@ -816,10 +944,8 @@ class TypePass:private Dispatcher {
 	  Swig_warning(WARN_PARSE_USING_UNDEF, Getfile(n), Getline(n), "Nothing known about '%s'.\n", SwigType_namestr(Getattr(n, "uname")));
 	}
       } else {
-
 	/* Only a single symbol is being used.  There are only a few symbols that
 	   we actually care about.  These are typedef, class declarations, and enum */
-
 	String *ntype = nodeType(ns);
 	if (Strcmp(ntype, "cdecl") == 0) {
 	  if (checkAttribute(ns, "storage", "typedef")) {
@@ -841,9 +967,15 @@ class TypePass:private Dispatcher {
 			|| (Getattr(c, "feature:extend") && !Getattr(c, "code"))
 			|| GetFlag(c, "feature:ignore"))) {
 
+		    /* Don't generate a method if the method is overridden in this class, 
+		     * for example don't generate another m(bool) should there be a Base::m(bool) :
+		     * struct Derived : Base { 
+		     *   void m(bool);
+		     *   using Base::m;
+		     * };
+		     */
 		    String *csymname = Getattr(c, "sym:name");
 		    if (!csymname || (Strcmp(csymname, symname) == 0)) {
-		      /* Check for existence in overload list already */
 		      {
 			String *decl = Getattr(c, "decl");
 			Node *over = Getattr(n, "sym:overloaded");
@@ -918,6 +1050,75 @@ class TypePass:private Dispatcher {
 		  }
 		}
 	      }
+
+	      /* Hack the parse tree symbol table for overloaded methods. Replace the "using" node with the
+	       * list of overloaded methods we have just added in as child nodes to the "using" node.
+	       * The node will still exist, it is just the symbol table linked list of overloaded methods
+	       * which is hacked. */
+	      if (Getattr(n, "sym:overloaded"))
+	      {
+#ifdef DEBUG_OVERLOADED
+show_overloaded(n);
+#endif
+		int cnt = 0;
+		Node *debugnode = n;
+		if (!firstChild(n)) {
+		  // Remove from overloaded list ('using' node does not actually end up adding in any methods)
+		  Node *ps = Getattr(n, "sym:previousSibling");
+		  Node *ns = Getattr(n, "sym:nextSibling");
+		  if (ps) {
+		    Setattr(ps, "sym:nextSibling", ns);
+		  }
+		  if (ns) {
+		    Setattr(ns, "sym:previousSibling", ps);
+		  }
+		} else {
+		  // The 'using' node results in methods being added in - slot in the these methods here 
+		  Node *ps = Getattr(n, "sym:previousSibling");
+		  Node *ns = Getattr(n, "sym:nextSibling");
+		  Node *fc = firstChild(n);
+		  Node *pp = fc;
+
+		  Node *firstoverloaded = Getattr(n, "sym:overloaded");
+		  if (firstoverloaded == n) {
+		    // This 'using' node we are cutting out was the first node in the overloaded list. 
+		    // Change the first node in the list to its first sibling
+		    Delattr(firstoverloaded, "sym:overloaded");
+		    Node *nnn = Getattr(firstoverloaded, "sym:nextSibling");
+		    firstoverloaded = fc;
+		    while (nnn) {
+		      Setattr(nnn, "sym:overloaded", firstoverloaded);
+		      nnn = Getattr(nnn, "sym:nextSibling");
+		    }
+		  }
+		  while (pp) {
+		    Node *ppn = Getattr(pp, "sym:nextSibling");
+		    Setattr(pp, "sym:overloaded", firstoverloaded);
+		    Setattr(pp, "sym:overname", NewStringf("%s_%d", Getattr(n, "sym:overname"), cnt++));
+		    if (ppn)
+		      pp = ppn;
+		    else
+		      break;
+		  }
+		  if (ps) {
+		    Setattr(ps, "sym:nextSibling", fc);
+		    Setattr(fc, "sym:previousSibling", ps);
+		  }
+		  if (ns) {
+		    Setattr(ns, "sym:previousSibling", pp);
+		    Setattr(pp, "sym:nextSibling", ns);
+		  }
+		  debugnode = firstoverloaded;
+		}
+		Delattr(n, "sym:previousSibling");
+		Delattr(n, "sym:nextSibling");
+		Delattr(n, "sym:overloaded");
+		Delattr(n, "sym:overname");
+#ifdef DEBUG_OVERLOADED
+show_overloaded(debugnode);
+#endif
+		clean_overloaded(n); // Needed?
+	      }
 	    }
 	  }
 	} else if ((Strcmp(ntype, "class") == 0) || ((Strcmp(ntype, "classforward") == 0))) {
