@@ -1,6 +1,10 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * cpp.c
  *
@@ -13,7 +17,7 @@
  * - Lines beginning with %# are stripped down to #... and passed through.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cpp_c[] = "$Id: cpp.c 10540 2008-06-21 15:23:02Z wsfulton $";
+char cvsroot_cpp_c[] = "$Id: cpp.c 12034 2010-05-21 07:10:12Z olly $";
 
 #include "swig.h"
 #include "preprocessor.h"
@@ -74,7 +78,7 @@ static void copy_location(const DOH *s1, DOH *s2) {
   Setline(s2, Getline((DOH *) s1));
 }
 
-static String *cpp_include(String_or_char *fn, int sysfile) {
+static String *cpp_include(const_String_or_char_ptr fn, int sysfile) {
   String *s = sysfile ? Swig_include_sys(fn) : Swig_include(fn);
   if (s && single_include) {
     String *file = Getfile(s);
@@ -85,7 +89,8 @@ static String *cpp_include(String_or_char *fn, int sysfile) {
     Setattr(included_files, file, file);
   }
   if (!s) {
-    Seek(fn, 0, SEEK_SET);
+    /* XXX(bhy) may not need the seek */
+    /* Seek(fn, 0, SEEK_SET); */
     if (ignore_missing) {
       Swig_warning(WARN_PP_MISSING_FILE, Getfile(fn), Getline(fn), "Unable to find '%s'\n", fn);
     } else {
@@ -186,7 +191,7 @@ void Preprocessor_init(void) {
   Preprocessor_expr_init();	/* Initialize the expression evaluator */
   included_files = NewHash();
 
-  id_scan = NewScanner();;
+  id_scan = NewScanner();
 
 }
 
@@ -261,8 +266,9 @@ void Preprocessor_error_as_warning(int a) {
  * ----------------------------------------------------------------------------- */
 
 
-String_or_char *Macro_vararg_name(String_or_char *str, String_or_char *line) {
-  String_or_char *argname, *varargname;
+String *Macro_vararg_name(const_String_or_char_ptr str, const_String_or_char_ptr line) {
+  String *argname;
+  String *varargname;
   char *s, *dots;
 
   argname = Copy(str);
@@ -288,24 +294,24 @@ String_or_char *Macro_vararg_name(String_or_char *str, String_or_char *line) {
   return varargname;
 }
 
-Hash *Preprocessor_define(const String_or_char *_str, int swigmacro) {
+Hash *Preprocessor_define(const_String_or_char_ptr _str, int swigmacro) {
   String *macroname = 0, *argstr = 0, *macrovalue = 0, *file = 0, *s = 0;
   Hash *macro = 0, *symbols = 0, *m1;
   List *arglist = 0;
   int c, line;
   int varargs = 0;
-  String_or_char *str = (String_or_char *) _str;
+  String *str;
 
   assert(cpp);
-  assert(str);
+  assert(_str);
 
   /* First make sure that string is actually a string */
-  if (DohCheck(str)) {
-    s = Copy(str);
-    copy_location(str, s);
+  if (DohCheck(_str)) {
+    s = Copy(_str);
+    copy_location(_str, s);
     str = s;
   } else {
-    str = NewString((char *) str);
+    str = NewString((char *) _str);
   }
   Seek(str, 0, SEEK_SET);
   line = Getline(str);
@@ -532,7 +538,7 @@ macro_error:
  *
  * Undefines a macro.
  * ----------------------------------------------------------------------------- */
-void Preprocessor_undef(const String_or_char *str) {
+void Preprocessor_undef(const_String_or_char_ptr str) {
   Hash *symbols;
   assert(cpp);
   symbols = Getattr(cpp, kpp_symbols);
@@ -623,7 +629,7 @@ unterm:
 /* -----------------------------------------------------------------------------
  * DOH *get_filename(DOH *str)
  *
- * Read a filename from str.   A filename can be enclose in quotes, angle brackets,
+ * Read a filename from str.   A filename can be enclosed in quotes, angle brackets,
  * or bare.
  * ----------------------------------------------------------------------------- */
 
@@ -650,14 +656,8 @@ static String *get_filename(String *str, int *sysfile) {
     if (isspace(c))
       Ungetc(c, str);
   }
-#if defined(_WIN32) || defined(MACSWIG)
-  /* accept Unix path separator on non-Unix systems */
-  Replaceall(fn, "/", SWIG_FILE_DELIMITER);
-#endif
-#if defined(__CYGWIN__)
-  /* accept Windows path separator in addition to Unix path separator */
-  Replaceall(fn, "\\", SWIG_FILE_DELIMITER);
-#endif
+  Swig_filename_unescape(fn);
+  Swig_filename_correct(fn);
   Seek(fn, 0, SEEK_SET);
   return fn;
 }
@@ -1598,9 +1598,9 @@ String *Preprocessor_parse(String *s) {
 	  s1 = cpp_include(fn, sysfile);
 	  if (s1) {
 	    if (include_all)
-	      Printf(ns, "%%includefile \"%s\" [\n", Swig_last_file());
+	      Printf(ns, "%%includefile \"%s\" [\n", Swig_filename_escape(Swig_last_file()));
 	    else if (import_all) {
-	      Printf(ns, "%%importfile \"%s\" [\n", Swig_last_file());
+	      Printf(ns, "%%importfile \"%s\" [\n", Swig_filename_escape(Swig_last_file()));
 	      push_imported();
 	    }
 
@@ -1653,7 +1653,7 @@ String *Preprocessor_parse(String *s) {
       state = 0;
       break;
 
-      /* Swig directives  */
+      /* SWIG directives  */
     case 100:
       /* %{,%} block  */
       if (c == '{') {
@@ -1718,7 +1718,7 @@ String *Preprocessor_parse(String *s) {
     case 110:
       if (!isidchar(c)) {
 	Ungetc(c, s);
-	/* Look for common Swig directives  */
+	/* Look for common SWIG directives  */
 	if (Equal(decl, kpp_dinclude) || Equal(decl, kpp_dimport) || Equal(decl, kpp_dextern)) {
 	  /* Got some kind of file inclusion directive  */
 	  if (allow) {
@@ -1737,7 +1737,7 @@ String *Preprocessor_parse(String *s) {
 	      char *dirname;
 	      add_chunk(ns, chunk, allow);
 	      copy_location(s, chunk);
-	      Printf(ns, "%sfile%s \"%s\" [\n", decl, opt, Swig_last_file());
+	      Printf(ns, "%sfile%s \"%s\" [\n", decl, opt, Swig_filename_escape(Swig_last_file()));
 	      if (Equal(decl, kpp_dimport)) {
 		push_imported();
 	      }
