@@ -1,6 +1,10 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * cwrap.c
  *
@@ -8,7 +12,7 @@
  * the naming of local variables, calling conventions, and so forth.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cwrap_c[] = "$Id: cwrap.c 11127 2009-02-13 07:26:06Z wsfulton $";
+char cvsroot_cwrap_c[] = "$Id: cwrap.c 11896 2010-03-04 21:27:23Z wsfulton $";
 
 #include "swig.h"
 
@@ -195,7 +199,11 @@ int Swig_cargs(Wrapper *w, ParmList *p) {
       String *type = Getattr(p, "type");
       /* default values only emitted if in compact default args mode */
       String *pvalue = (compactdefargs) ? Getattr(p, "value") : 0;
-      SwigType *altty = SwigType_alttype(type, 0);
+
+      /* When using compactdefaultargs, the code generated initialises a variable via a constructor call that accepts the
+       * default value as a parameter. The default constructor is not called and therefore SwigValueWrapper is not needed. */
+      SwigType *altty = pvalue ? 0 : SwigType_alttype(type, 0);
+
       int tycode = SwigType_type(type);
       if (tycode == T_REFERENCE) {
 	if (pvalue) {
@@ -774,7 +782,7 @@ int Swig_add_extension_code(Node *n, const String *function_name, ParmList *parm
  * Converts a C++ method node to a function accessor function.
  * ----------------------------------------------------------------------------- */
 
-int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *director_type, int is_director) {
+int Swig_MethodToFunction(Node *n, const_String_or_char_ptr nspace, String *classname, int flags, SwigType *director_type, int is_director) {
   String *name, *qualifier;
   ParmList *parms;
   SwigType *type;
@@ -799,7 +807,7 @@ int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *direc
     SwigType_push(type, qualifier);
   }
   SwigType_add_pointer(type);
-  p = NewParm(type, "self");
+  p = NewParm(type, "self", n);
   Setattr(p, "self", "1");
   Setattr(p, "hidden","1");
   /*
@@ -888,7 +896,7 @@ int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *direc
     String *defaultargs = Getattr(n, "defaultargs");
     String *code = Getattr(n, "code");
     String *cname = Getattr(n, "classname") ? Getattr(n, "classname") : classname;
-    String *membername = Swig_name_member(cname, name);
+    String *membername = Swig_name_member(nspace, cname, name);
     String *mangled = Swig_name_mangle(membername);
     int is_smart_pointer = flags & CWRAP_SMART_POINTER;
 
@@ -1001,7 +1009,7 @@ Node *Swig_directormap(Node *module, String *type) {
  * This function creates a C wrapper for a C constructor function. 
  * ----------------------------------------------------------------------------- */
 
-int Swig_ConstructorToFunction(Node *n, String *classname, String *none_comparison, String *director_ctor, int cplus, int flags) {
+int Swig_ConstructorToFunction(Node *n, const_String_or_char_ptr nspace, String *classname, String *none_comparison, String *director_ctor, int cplus, int flags) {
   ParmList *parms;
   Parm *prefix_args;
   Parm *p;
@@ -1041,7 +1049,7 @@ int Swig_ConstructorToFunction(Node *n, String *classname, String *none_comparis
     String *cres;
     String *defaultargs = Getattr(n, "defaultargs");
     String *code = Getattr(n, "code");
-    String *membername = Swig_name_construct(classname);
+    String *membername = Swig_name_construct(nspace, classname);
     String *mangled = Swig_name_mangle(membername);
 
     /* Check if the constructor is overloaded.   If so, and it has code attached, we append an extra suffix
@@ -1145,13 +1153,13 @@ int Swig_ConstructorToFunction(Node *n, String *classname, String *none_comparis
  * This function creates a C wrapper for a destructor function.
  * ----------------------------------------------------------------------------- */
 
-int Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags) {
+int Swig_DestructorToFunction(Node *n, const_String_or_char_ptr nspace, String *classname, int cplus, int flags) {
   SwigType *type;
   Parm *p;
 
   type = NewString(classname);
   SwigType_add_pointer(type);
-  p = NewParm(type, "self");
+  p = NewParm(type, "self", n);
   Setattr(p, "self", "1");
   Setattr(p, "hidden", "1");
   Setattr(p, "wrap:disown", "1");
@@ -1162,7 +1170,7 @@ int Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
     String *cres;
     String *call;
     String *membername, *mangled, *code;
-    membername = Swig_name_destroy(classname);
+    membername = Swig_name_destroy(nspace, classname);
     mangled = Swig_name_mangle(membername);
     code = Getattr(n, "code");
     if (code) {
@@ -1203,7 +1211,7 @@ int Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
  * This function creates a C wrapper for setting a structure member.
  * ----------------------------------------------------------------------------- */
 
-int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **call) {
+int Swig_MembersetToFunction(Node *n, String *classname, int flags) {
   String *name;
   ParmList *parms;
   Parm *p;
@@ -1211,10 +1219,7 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   SwigType *ty;
   SwigType *type;
   SwigType *void_type = NewString("void");
-  String *membername;
-  String *mangled;
   String *self = 0;
-  String *sname;
 
   int varcref = flags & CWRAP_NATURAL_VAR;
 
@@ -1228,19 +1233,15 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   name = Getattr(n, "name");
   type = Getattr(n, "type");
 
-  sname = Swig_name_set(name);
-  membername = Swig_name_member(classname, sname);
-  mangled = Swig_name_mangle(membername);
-
   t = NewString(classname);
   SwigType_add_pointer(t);
-  parms = NewParm(t, "self");
+  parms = NewParm(t, "self", n);
   Setattr(parms, "self", "1");
   Setattr(parms, "hidden","1");
   Delete(t);
 
   ty = Swig_wrapped_member_var_type(type, varcref);
-  p = NewParm(ty, name);
+  p = NewParm(ty, name, n);
   Setattr(parms, "hidden", "1");
   set_nextSibling(parms, p);
 
@@ -1251,21 +1252,32 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   Delete(p);
 
   if (flags & CWRAP_EXTEND) {
+    String *call;
     String *cres;
     String *code = Getattr(n, "code");
+
+    String *sname = Swig_name_set(0, name);
+    String *membername = Swig_name_member(0, classname, sname);
+    String *mangled = Swig_name_mangle(membername);
+
     if (code) {
       /* I don't think this ever gets run - WSF */
       Swig_add_extension_code(n, mangled, parms, void_type, code, cparse_cplusplus, "self");
     }
-    *call = Swig_cfunction_call(mangled, parms);
-    cres = NewStringf("%s;", *call);
+    call = Swig_cfunction_call(mangled, parms);
+    cres = NewStringf("%s;", call);
     Setattr(n, "wrap:action", cres);
+
     Delete(cres);
+    Delete(call);
+    Delete(mangled);
+    Delete(membername);
+    Delete(sname);
   } else {
-    String *cres;
-    *call = Swig_cmemberset_call(name, type, self, varcref);
-    cres = NewStringf("%s;", *call);
+    String *call = Swig_cmemberset_call(name, type, self, varcref);
+    String *cres = NewStringf("%s;", call);
     Setattr(n, "wrap:action", cres);
+    Delete(call);
     Delete(cres);
   }
   Setattr(n, "type", void_type);
@@ -1273,9 +1285,6 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   Delete(parms);
   Delete(ty);
   Delete(void_type);
-  Delete(membername);
-  Delete(sname);
-  Delete(mangled);
   Delete(self);
   return SWIG_OK;
 }
@@ -1292,10 +1301,7 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   SwigType *t;
   SwigType *ty;
   SwigType *type;
-  String *membername;
-  String *mangled;
   String *self = 0;
-  String *gname;
 
   int varcref = flags & CWRAP_NATURAL_VAR;
 
@@ -1315,13 +1321,9 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   name = Getattr(n, "name");
   type = Getattr(n, "type");
 
-  gname = Swig_name_get(name);
-  membername = Swig_name_member(classname, gname);
-  mangled = Swig_name_mangle(membername);
-
   t = NewString(classname);
   SwigType_add_pointer(t);
-  parms = NewParm(t, "self");
+  parms = NewParm(t, "self", n);
   Setattr(parms, "self", "1");
   Setattr(parms, "hidden","1");
   Delete(t);
@@ -1330,8 +1332,12 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   if (flags & CWRAP_EXTEND) {
     String *call;
     String *cres;
-
     String *code = Getattr(n, "code");
+
+    String *gname = Swig_name_get(0, name);
+    String *membername = Swig_name_member(0, classname, gname);
+    String *mangled = Swig_name_mangle(membername);
+
     if (code) {
       /* I don't think this ever gets run - WSF */
       Swig_add_extension_code(n, mangled, parms, ty, code, cparse_cplusplus, "self");
@@ -1339,8 +1345,12 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
     call = Swig_cfunction_call(mangled, parms);
     cres = Swig_cresult(ty, "result", call);
     Setattr(n, "wrap:action", cres);
+
     Delete(cres);
     Delete(call);
+    Delete(mangled);
+    Delete(membername);
+    Delete(gname);
   } else {
     String *call = Swig_cmemberget_call(name, type, self, varcref);
     String *cres = Swig_cresult(ty, "result", call);
@@ -1352,9 +1362,6 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   Setattr(n, "parms", parms);
   Delete(parms);
   Delete(ty);
-  Delete(membername);
-  Delete(gname);
-  Delete(mangled);
 
   return SWIG_OK;
 }
@@ -1377,10 +1384,10 @@ int Swig_VarsetToFunction(Node *n, int flags) {
   type = Getattr(n, "type");
   nname = SwigType_namestr(name);
   ty = Swig_wrapped_var_type(type, varcref);
-  parms = NewParm(ty, name);
+  parms = NewParm(ty, name, n);
 
   if (flags & CWRAP_EXTEND) {
-    String *sname = Swig_name_set(name);
+    String *sname = Swig_name_set(0, name);
     String *mangled = Swig_name_mangle(sname);
     String *call = Swig_cfunction_call(mangled, parms);
     String *cres = NewStringf("%s;", call);
@@ -1434,7 +1441,7 @@ int Swig_VargetToFunction(Node *n, int flags) {
   ty = Swig_wrapped_var_type(type, varcref);
 
   if (flags & CWRAP_EXTEND) {
-    String *sname = Swig_name_get(name);
+    String *sname = Swig_name_get(0, name);
     String *mangled = Swig_name_mangle(sname);
     call = Swig_cfunction_call(mangled, 0);
     cres = Swig_cresult(ty, "result", call);
