@@ -1,6 +1,10 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * stype.c
  *
@@ -8,7 +12,7 @@
  * the form of simple strings.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_stype_c[] = "$Id: stype.c 10254 2008-02-13 23:25:37Z wsfulton $";
+char cvsroot_stype_c[] = "$Id: stype.c 12007 2010-05-02 21:35:02Z wsfulton $";
 
 #include "swig.h"
 #include "cparse.h"
@@ -271,223 +275,187 @@ int SwigType_issimple(SwigType *t) {
 }
 
 /* -----------------------------------------------------------------------------
- * SwigType_default()
+ * SwigType_default_create()
  *
- * Create the default string for this datatype.   This takes a type and strips it
- * down to its most primitive form--resolving all typedefs and removing operators.
+ * Create the default type for this datatype. This takes a type and strips it
+ * down to a generic form first by resolving all typedefs.
  *
  * Rules:
- *     Pointers:      p.SWIGTYPE
- *     References:    r.SWIGTYPE
- *     Arrays:        a().SWIGTYPE
- *     Types:         SWIGTYPE
- *     MemberPointer: m(CLASS).SWIGTYPE
- *     Enums:         enum SWIGTYPE
+ *     Pointers:              p.SWIGTYPE
+ *     References:            r.SWIGTYPE
+ *     Arrays no dimension:   a().SWIGTYPE
+ *     Arrays with dimension: a(ANY).SWIGTYPE
+ *     Member pointer:        m(CLASS).SWIGTYPE
+ *     Function pointer:      f(ANY).SWIGTYPE
+ *     Enums:                 enum SWIGTYPE
+ *     Types:                 SWIGTYPE
  *
- * Note: if this function is applied to a primitive type, it returns NULL.  This
- * allows recursive application for special types like arrays.
+ * Examples (also see SwigType_default_deduce):
+ *
+ *  int [2][4]
+ *    a(2).a(4).int
+ *    a(ANY).a(ANY).SWIGTYPE
+ *
+ *  struct A {};
+ *  typedef A *Aptr;
+ *  Aptr const &
+ *    r.q(const).Aptr
+ *    r.q(const).p.SWIGTYPE
+ *
+ *  enum E {e1, e2};
+ *  enum E const &
+ *    r.q(const).enum E
+ *    r.q(const).enum SWIGTYPE
  * ----------------------------------------------------------------------------- */
 
-#ifdef SWIG_DEFAULT_CACHE
-static Hash *default_cache = 0;
-#endif
+SwigType *SwigType_default_create(SwigType *ty) {
+  SwigType *r = 0;
+  List *l;
+  Iterator it;
+  int numitems;
 
-#define SWIG_NEW_TYPE_DEFAULT
-/* The new default type resolution method:
+  if (!SwigType_isvarargs(ty)) {
+    SwigType *t = SwigType_typedef_resolve_all(ty);
+    r = NewStringEmpty();
+    l = SwigType_split(t);
+    numitems = Len(l);
 
-1.- It preserves the original mixed types, then it goes 'backward'
-    first deleting the qualifier, then the inner types
-    
-    typedef A *Aptr;
-    const Aptr&;
-    r.q(const).Aptr       -> r.q(const).p.SWIGTYPE
-    r.q(const).p.SWIGTYPE -> r.p.SWIGTYPE
-    r.p.SWIGTYPE          -> r.SWIGTYPE
-    r.SWIGTYPE            -> SWIGTYPE
+    if (numitems >= 1) {
+      String *last_subtype = Getitem(l, numitems-1);
+      if (SwigType_isenum(last_subtype))
+	Setitem(l, numitems-1, NewString("enum SWIGTYPE"));
+      else
+	Setitem(l, numitems-1, NewString("SWIGTYPE"));
+    }
 
+    for (it = First(l); it.item; it = Next(it)) {
+      String *subtype = it.item;
+      if (SwigType_isarray(subtype)) {
+	if (Equal(subtype, "a()."))
+	  Append(r, NewString("a()."));
+	else
+	  Append(r, NewString("a(ANY)."));
+      } else if (SwigType_isfunction(subtype)) {
+	Append(r, NewString("f(ANY).SWIGTYPE"));
+	break;
+      } else if (SwigType_ismemberpointer(subtype)) {
+	Append(r, NewString("m(CLASS).SWIGTYPE"));
+	break;
+      } else {
+	Append(r, subtype);
+      }
+    }
 
-    enum Hello {};
-    const Hello& hi;
-    r.q(const).Hello          -> r.q(const).enum SWIGTYPE
-    r.q(const).enum SWIGTYPE  -> r.enum SWIGTYPE
-    r.enum SWIGTYPE           -> r.SWIGTYPE
-    r.SWIGTYPE                -> SWIGTYPE
+    Delete(l);
+    Delete(t);
+  }
 
-    int a[2][4];
-    a(2).a(4).int           -> a(ANY).a(ANY).SWIGTYPE
-    a(ANY).a(ANY).SWIGTYPE  -> a(ANY).a().SWIGTYPE
-    a(ANY).a().SWIGTYPE     -> a(ANY).p.SWIGTYPE
-    a(ANY).p.SWIGTYPE       -> a(ANY).SWIGTYPE
-    a(ANY).SWIGTYPE         -> a().SWIGTYPE
-    a().SWIGTYPE            -> p.SWIGTYPE
-    p.SWIGTYPE              -> SWIGTYPE
-*/
+  return r;
+}
 
-static
-void SwigType_add_default(String *def, SwigType *nr) {
-  if (Strcmp(nr, "SWIGTYPE") == 0) {
-    Append(def, "SWIGTYPE");
-  } else {
-    String *q = SwigType_isqualifier(nr) ? SwigType_pop(nr) : 0;
-    if (q && strstr(Char(nr), "SWIGTYPE")) {
-      Append(def, nr);
-    } else {
-      String *nd = SwigType_default(nr);
-      if (nd) {
-	String *bdef = nd;
-	if (q) {
-	  bdef = NewStringf("%s%s", q, nd);
-	  if ((Strcmp(nr, bdef) == 0)) {
-	    Delete(bdef);
-	    bdef = nd;
+/* -----------------------------------------------------------------------------
+ * SwigType_default_deduce()
+ *
+ * This function implements type deduction used in the typemap matching rules
+ * and is very close to the type deduction used in partial template class
+ * specialization matching in that the most specialized type is always chosen.
+ * SWIGTYPE is used as the generic type. The basic idea is to repeatedly call
+ * this function to find a deduced type unless until nothing matches.
+ *
+ * The type t must have already been converted to the default type via a call to
+ * SwigType_default_create() before calling this function.
+ *
+ * Example deductions (matching the examples described in SwigType_default_create),
+ * where the the most specialized matches are highest in the list:
+ *
+ *    a(ANY).a(ANY).SWIGTYPE
+ *    a(ANY).a().SWIGTYPE
+ *    a(ANY).p.SWIGTYPE
+ *    a(ANY).SWIGTYPE
+ *    a().SWIGTYPE
+ *    p.SWIGTYPE
+ *    SWIGTYPE
+ *
+ *    r.q(const).p.SWIGTYPE
+ *    r.q(const).SWIGTYPE
+ *    r.SWIGTYPE
+ *    SWIGTYPE
+ *
+ *    r.q(const).enum SWIGTYPE
+ *    r.enum SWIGTYPE
+ *    r.SWIGTYPE
+ *    SWIGTYPE
+ * ----------------------------------------------------------------------------- */
+
+SwigType *SwigType_default_deduce(SwigType *t) {
+  SwigType *r = NewStringEmpty();
+  List *l;
+  Iterator it;
+  int numitems;
+
+  l = SwigType_split(t);
+
+  numitems = Len(l);
+  if (numitems >= 1) {
+    String *last_subtype = Getitem(l, numitems-1);
+    int is_enum = SwigType_isenum(last_subtype);
+
+    if (numitems >=2 ) {
+      String *subtype = Getitem(l, numitems-2); /* last but one */
+      if (SwigType_isarray(subtype)) {
+	if (is_enum) {
+	  /* enum deduction, enum SWIGTYPE => SWIGTYPE */
+	  Setitem(l, numitems-1, NewString("SWIGTYPE"));
+	} else {
+	  /* array deduction, a(ANY). => a(). => p. */
+	  String *deduced_subtype = 0;
+	  if (Strcmp(subtype, "a().") == 0) {
+	    deduced_subtype = NewString("p.");
+	  } else if (Strcmp(subtype, "a(ANY).") == 0) {
+	    deduced_subtype = NewString("a().");
 	  } else {
-	    Delete(nd);
+	    assert(0);
 	  }
+	  Setitem(l, numitems-2, deduced_subtype);
 	}
-	Append(def, bdef);
-	Delete(bdef);
+      } else if (SwigType_ismemberpointer(subtype)) {
+	/* member pointer deduction, m(CLASS). => p. */
+	Setitem(l, numitems-2, NewString("p."));
+      } else if (is_enum && !SwigType_isqualifier(subtype)) {
+	/* enum deduction, enum SWIGTYPE => SWIGTYPE */
+	Setitem(l, numitems-1, NewString("SWIGTYPE"));
       } else {
-	Append(def, nr);
+	/* simple type deduction, eg, r.p.p. => r.p. */
+	/* also function pointers eg, p.f(ANY). => p. */
+	Delitem(l, numitems-2);
       }
-    }
-    Delete(q);
-  }
-}
-
-
-SwigType *SwigType_default(SwigType *t) {
-  String *r1, *def;
-  String *r = 0;
-  char *cr;
-
-#ifdef SWIG_DEFAULT_CACHE
-  if (!default_cache)
-    default_cache = NewHash();
-
-  r = Getattr(default_cache, t);
-  if (r) {
-    return Copy(r);
-  }
-#endif
-
-  if (SwigType_isvarargs(t)) {
-    return 0;
-  }
-
-  r = t;
-  while ((r1 = SwigType_typedef_resolve(r))) {
-    if (r != t)
-      Delete(r);
-    r = r1;
-  }
-  if (SwigType_isqualifier(r)) {
-    String *q;
-    if (r == t)
-      r = Copy(t);
-    q = SwigType_pop(r);
-    if (strstr(Char(r), "SWIGTYPE")) {
-      Delete(q);
-      def = r;
-      return def;
-    }
-    Delete(q);
-  }
-  cr = Char(r);
-  if (strcmp(cr, "p.SWIGTYPE") == 0) {
-    def = NewString("SWIGTYPE");
-  } else if (SwigType_ispointer(r)) {
-#ifdef SWIG_NEW_TYPE_DEFAULT
-    SwigType *nr = Copy(r);
-    SwigType_del_pointer(nr);
-    def = SwigType_isfunction(nr) ? NewStringEmpty() : NewString("p.");
-    SwigType_add_default(def, nr);
-    Delete(nr);
-#else
-    def = NewString("p.SWIGTYPE");
-#endif
-  } else if (strcmp(cr, "r.SWIGTYPE") == 0) {
-    def = NewString("SWIGTYPE");
-  } else if (SwigType_isreference(r)) {
-#ifdef SWIG_NEW_TYPE_DEFAULT
-    SwigType *nr = Copy(r);
-    SwigType_del_reference(nr);
-    def = NewString("r.");
-    SwigType_add_default(def, nr);
-    Delete(nr);
-#else
-    def = NewString("r.SWIGTYPE");
-#endif
-  } else if (SwigType_isarray(r)) {
-    if (strcmp(cr, "a().SWIGTYPE") == 0) {
-      def = NewString("p.SWIGTYPE");
-    } else if (strcmp(cr, "a(ANY).SWIGTYPE") == 0) {
-      def = NewString("a().SWIGTYPE");
     } else {
-      int i, empty = 0;
-      int ndim = SwigType_array_ndim(r);
-      SwigType *nr = Copy(r);
-      for (i = 0; i < ndim; i++) {
-	String *dim = SwigType_array_getdim(r, i);
-	if (!Len(dim)) {
-	  char *c = Char(nr);
-	  empty = strstr(c, "a(ANY).") != c;
-	}
-	Delete(dim);
-      }
-      if (empty) {
-	def = NewString("a().");
+      if (is_enum) {
+	/* enum deduction, enum SWIGTYPE => SWIGTYPE */
+	Setitem(l, numitems-1, NewString("SWIGTYPE"));
       } else {
-	def = NewString("a(ANY).");
+	/* delete the only item, we are done with deduction */
+	Delitem(l, 0);
       }
-#ifdef SWIG_NEW_TYPE_DEFAULT
-      SwigType_del_array(nr);
-      SwigType_add_default(def, nr);
-#else
-      Append(def, "SWIGTYPE");
-#endif
-      Delete(nr);
-    }
-  } else if (SwigType_ismemberpointer(r)) {
-    if (strcmp(cr, "m(CLASS).SWIGTYPE") == 0) {
-      def = NewString("p.SWIGTYPE");
-    } else {
-      def = NewString("m(CLASS).SWIGTYPE");
-    }
-  } else if (SwigType_isenum(r)) {
-    if (strcmp(cr, "enum SWIGTYPE") == 0) {
-      def = NewString("SWIGTYPE");
-    } else {
-      def = NewString("enum SWIGTYPE");
-    }
-  } else if (SwigType_isfunction(r)) {
-    if (strcmp(cr, "f(ANY).SWIGTYPE") == 0) {
-      def = NewString("p.SWIGTYPE");
-    } else {
-      def = NewString("p.f(ANY).SWIGTYPE");
     }
   } else {
-    def = NewString("SWIGTYPE");
+    assert(0);
   }
-  if (r != t)
+
+  for (it = First(l); it.item; it = Next(it)) {
+    Append(r, it.item);
+  }
+
+  if (Len(r) == 0) {
     Delete(r);
-  if (Equal(def, t)) {
-    Delete(def);
-    def = 0;
+    r = 0;
   }
-#ifdef SWIG_DEFAULT_CACHE
-  /* The cache produces strange results, see enum_template.i case */
-  if (def) {
-    String *cdef = Copy(def);
-    Setattr(default_cache, t, cdef);
-    Delete(cdef);
-  }
-#endif
 
-  /* Printf(stderr,"type : def %s : %s\n", t, def);  */
-
-  return def;
+  Delete(l);
+  return r;
 }
 
+
 /* -----------------------------------------------------------------------------
  * SwigType_namestr()
  *
@@ -525,7 +493,13 @@ String *SwigType_namestr(const SwigType *t) {
   Putc(' ', r);
   Putc('>', r);
   suffix = SwigType_templatesuffix(t);
-  Append(r, suffix);
+  if (Len(suffix) > 0) {
+    String *suffix_namestr = SwigType_namestr(suffix);
+    Append(r, suffix_namestr);
+    Delete(suffix_namestr);
+  } else {
+    Append(r, suffix);
+  }
   Delete(suffix);
   Delete(p);
   return r;
@@ -537,14 +511,17 @@ String *SwigType_namestr(const SwigType *t) {
  * Create a C string representation of a datatype.
  * ----------------------------------------------------------------------------- */
 
-String *SwigType_str(SwigType *s, const String_or_char *id) {
+String *SwigType_str(SwigType *s, const_String_or_char_ptr id) {
   String *result;
   String *element = 0, *nextelement;
   List *elements;
   int nelements, i;
 
   if (id) {
-    result = NewString(id);
+    /* stringify the id expanding templates, for example when the id is a fully qualified templated class name */
+    String *id_str = NewString(id); /* unfortunate copy due to current const limitations */
+    result = SwigType_str(id_str, 0);
+    Delete(id_str);
   } else {
     result = NewStringEmpty();
   }
@@ -732,7 +709,7 @@ SwigType *SwigType_ltype(SwigType *s) {
  *          with an equivalent assignable version.
  * -------------------------------------------------------------------- */
 
-String *SwigType_lstr(SwigType *s, const String_or_char *id) {
+String *SwigType_lstr(SwigType *s, const_String_or_char_ptr id) {
   String *result;
   SwigType *tc;
 
@@ -749,7 +726,7 @@ String *SwigType_lstr(SwigType *s, const String_or_char *id) {
  * datatype printed by str().
  * ----------------------------------------------------------------------------- */
 
-String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
+String *SwigType_rcaststr(SwigType *s, const_String_or_char_ptr name) {
   String *result, *cast;
   String *element = 0, *nextelement;
   SwigType *td, *rs, *tc = 0;
@@ -892,7 +869,7 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
  * Casts a variable from the real type to the local datatype.
  * ----------------------------------------------------------------------------- */
 
-String *SwigType_lcaststr(SwigType *s, const String_or_char *name) {
+String *SwigType_lcaststr(SwigType *s, const_String_or_char_ptr name) {
   String *result;
 
   result = NewStringEmpty();
@@ -1085,6 +1062,25 @@ void SwigType_typename_replace(SwigType *t, String *pat, String *rep) {
   Delete(elem);
 }
 
+/* -----------------------------------------------------------------------------
+ * SwigType_remove_global_scope_prefix()
+ *
+ * Removes the unary scope operator (::) prefix indicating global scope in all 
+ * components of the type
+ * ----------------------------------------------------------------------------- */
+
+SwigType *SwigType_remove_global_scope_prefix(const SwigType *t) {
+  SwigType *result;
+  const char *type = Char(t);
+  if (strncmp(type, "::", 2) == 0)
+    type += 2;
+  result = NewString(type);
+  Replaceall(result, ".::", ".");
+  Replaceall(result, "(::", "(");
+  Replaceall(result, "enum ::", "enum ");
+  return result;
+}
+
 /* -----------------------------------------------------------------------------
  * SwigType_check_decl()
  *
