@@ -1,21 +1,23 @@
 /* ----------------------------------------------------------------------------- 
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * swigscan.h
  *
  * C/C++ scanner. 
  * ----------------------------------------------------------------------------- */
 
-/* $Id: swig.h 9633 2007-01-10 23:43:07Z beazley $ */
-
 typedef struct Scanner Scanner;
 
-extern Scanner     *NewScanner();
+extern Scanner     *NewScanner(void);
 extern void         DelScanner(Scanner *);
 extern void         Scanner_clear(Scanner *);
 extern void         Scanner_push(Scanner *, String *);
-extern void         Scanner_pushtoken(Scanner *, int, const String_or_char *value);
+extern void         Scanner_pushtoken(Scanner *, int, const_String_or_char_ptr value);
 extern int          Scanner_token(Scanner *);
 extern String      *Scanner_text(Scanner *);
 extern void         Scanner_skip_line(Scanner *);
@@ -64,6 +66,7 @@ extern void         Scanner_freeze_line(Scanner *s, int val);
 #define   SWIG_TOKEN_ULONGLONG    29       /* 314ULL */
 #define   SWIG_TOKEN_QUESTION     30       /* ? */
 #define   SWIG_TOKEN_COMMENT      31       /* C or C++ comment */
+#define   SWIG_TOKEN_BOOL         32       /* true or false */
 #define   SWIG_TOKEN_ILLEGAL      99
 #define   SWIG_TOKEN_ERROR        -1
 
